@@ -1,26 +1,28 @@
 /*
- *  $Id: arrows.c,v 1.13 2004/12/20 20:42:58 tom Exp $
+ *  $Id: arrows.c,v 1.17 2005/11/28 00:21:25 tom Exp $
  *
  *  arrows.c -- draw arrows to indicate end-of-range for lists
  *
- * Copyright 2000-2003,2004   Thomas E. Dickey
+ * Copyright 2000-2004,2005   Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
  */
 
-#include "dialog.h"
+#include <dialog.h>
 
 #ifdef USE_WIDE_CURSES
 #define add_acs(win, code) wadd_wch(win, W ## code)
@@ -67,7 +69,7 @@ dlg_draw_arrows2(WINDOW *dialog,
     if (top_arrow) {
 	wattrset(dialog, merge_colors(uarrow_attr, attr));
 	(void) add_acs(dialog, ACS_UARROW);
-	(void) waddstr(dialog, "(+)");
+	(void) waddstr(dialog, "(-)");
     } else {
 	wattrset(dialog, attr);
 	(void) whline(dialog, ACS_HLINE, 4);
