@@ -1,51 +1,48 @@
 /*
- *  $Id: buttons.c,v 1.56 2004/12/20 20:42:58 tom Exp $
+ *  $Id: buttons.c,v 1.68 2005/12/07 01:43:56 tom Exp $
  *
  *  buttons.c -- draw buttons, e.g., OK/Cancel
  *
- * Copyright 2000-2003,2004	Thomas E. Dickey
+ * Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
  */
 
-#include "dialog.h"
-
-#ifdef USE_WIDE_CURSES
-#include <wctype.h>
-#define dlg_toupper(ch) towupper(ch)
-#define dlg_isupper(ch) iswupper(ch)
-#else
-#define dlg_toupper(ch) toupper(ch)
-#define dlg_isupper(ch) (isalpha(ch) && isupper(ch))
-#endif
+#include <dialog.h>
+#include <dlg_keys.h>
 
 #define MIN_BUTTON (dialog_state.visit_items ? -1 : 0)
 
 static void
 center_label(char *buffer, int longest, const char *label)
 {
-    int len = strlen(label);
+    int len = dlg_count_columns(label);
+    int left = 0, right = 0;
 
+    *buffer = 0;
     if (len < longest) {
-	if ((len = (longest - len) / 2) > 0) {
-	    longest -= len;
-	    sprintf(buffer, "%*s", len, " ");
-	    buffer += strlen(buffer);
-	}
+	left = (longest - len) / 2;
+	right = (longest - len - left);
+	if (left > 0)
+	    sprintf(buffer, "%*s", left, " ");
     }
-    sprintf(buffer, "%-*s", longest, label);
+    strcat(buffer, label);
+    if (right > 0)
+	sprintf(buffer + strlen(buffer), "%*s", right, " ");
 }
 
 /*
@@ -91,7 +88,7 @@ string_to_char(const char **stringp)
 static void
 print_button(WINDOW *win, const char *label, int y, int x, int selected)
 {
-    int i, j;
+    int i;
     int state = 0;
     const int *indx = dlg_index_wchars(label);
     int limit = dlg_count_wchars(label);
@@ -135,8 +132,7 @@ print_button(WINDOW *win, const char *label, int y, int x, int selected)
 	    state = 2;
 	    break;
 	}
-	for (j = first; j < last; ++j)
-	    (void) waddch(win, CharOf(label[j]));
+	waddnstr(win, label + first, last - first);
     }
     wattrset(win, selected
 	     ? button_active_attr
@@ -158,7 +154,8 @@ dlg_button_count(const char **labels)
 }
 
 /*
- * Compute the size of the button array.
+ * Compute the size of the button array in columns.  Return the total number of
+ * columns in *length, and the longest button's columns in *longest
  */
 void
 dlg_button_sizes(const char **labels,
@@ -175,7 +172,7 @@ dlg_button_sizes(const char **labels,
 	    *length += 1;
 	    *longest = 1;
 	} else {
-	    int len = strlen(labels[n]);
+	    int len = dlg_count_columns(labels[n]);
 	    if (len > *longest)
 		*longest = len;
 	    *length += len;
@@ -256,6 +253,7 @@ dlg_draw_buttons(WINDOW *win,
     int final_y;
     int gap;
     int margin;
+    unsigned need;
     char *buffer;
 
     dlg_mouse_setbase(getbegx(win), getbegy(win));
@@ -272,12 +270,22 @@ dlg_draw_buttons(WINDOW *win,
 	x += margin;
     }
 
-    buffer = malloc((unsigned) longest + 1);
+    /*
+     * Allocate a buffer big enough for any label.
+     */
+    need = longest;
+    for (n = 0; labels[n] != 0; ++n) {
+	need += strlen(labels[n]) + 1;
+    }
+    buffer = malloc(need);
     assert_ptr(buffer, "dlg_draw_buttons");
 
+    /*
+     * Draw the labels.
+     */
     for (n = 0; labels[n] != 0; n++) {
 	center_label(buffer, longest, labels[n]);
-	mouse_mkbutton(y, x, strlen(buffer), n);
+	mouse_mkbutton(y, x, dlg_count_columns(buffer), n);
 	print_button(win, buffer, y, x,
 		     (selected == n) || (n == 0 && selected < 0));
 	if (selected == n)
@@ -322,6 +330,24 @@ dlg_match_char(int ch, const char *string)
     return FALSE;
 }
 
+/*
+ * Find the first uppercase character in the label, which we may use for an
+ * abbreviation.
+ */
+int
+dlg_button_to_char(const char *label)
+{
+    int cmp = -1;
+
+    while (*label != 0) {
+	cmp = string_to_char(&label);
+	if (dlg_isupper(cmp)) {
+	    break;
+	}
+    }
+    return cmp;
+}
+
 /*
  * Given a list of button labels, and a character which may be the abbreviation
  * for one, find it, if it exists.  An abbreviation will be the first character
@@ -332,19 +358,13 @@ dlg_char_to_button(int ch, const char **labels)
 {
     if (labels != 0) {
 	int j;
-	const char *label;
 
 	ch = dlg_toupper(dlg_last_getc());
 	for (j = 0; labels[j] != 0; ++j) {
-	    label = labels[j];
-	    while (*label != 0) {
-		int cmp = string_to_char(&label);
-		if (ch == cmp) {
-		    dlg_flush_getc();
-		    return j;
-		} else if (dlg_isupper(cmp)) {
-		    break;
-		}
+	    int cmp = dlg_button_to_char(labels[j]);
+	    if (ch == cmp) {
+		dlg_flush_getc();
+		return j;
 	    }
 	}
     }
@@ -413,12 +433,40 @@ my_help_label(void)
 const char **
 dlg_exit_label(void)
 {
-    static const char *labels[3];
-    int n = 0;
+    const char **result;
 
-    labels[n++] = my_exit_label();
-    labels[n] = 0;
-    return labels;
+    if (dialog_vars.extra_button) {
+	result = dlg_ok_labels();
+    } else {
+	static const char *labels[3];
+	int n = 0;
+
+	labels[n++] = my_exit_label();
+	if (dialog_vars.help_button)
+	    labels[n++] = my_help_label();
+	labels[n] = 0;
+
+	result = labels;
+    }
+    return result;
+}
+
+/*
+ * Map the given button index for dlg_exit_labels() into our exit-code.
+ */
+int
+dlg_exit_buttoncode(int button)
+{
+    int result = DLG_EXIT_ERROR;
+
+    if (dialog_vars.extra_button) {
+	result = dlg_ok_buttoncode(button);
+    } else if (button == 0) {
+	result = DLG_EXIT_OK;
+    } else if (button == 1 && dialog_vars.help_button) {
+	result = DLG_EXIT_HELP;
+    }
+    return result;
 }
 
 const char **
@@ -428,6 +476,8 @@ dlg_ok_label(void)
     int n = 0;
 
     labels[n++] = my_ok_label();
+    if (dialog_vars.help_button)
+	labels[n++] = my_help_label();
     labels[n] = 0;
     return labels;
 }
@@ -528,13 +578,45 @@ dlg_defaultno_button(void)
 const char **
 dlg_yes_labels(void)
 {
-    static const char *labels[3];
-    int n = 0;
+    const char **result;
 
-    labels[n++] = my_yes_label();
-    labels[n++] = my_no_label();
-    labels[n] = 0;
-    return labels;
+    if (dialog_vars.extra_button) {
+	result = dlg_ok_labels();
+    } else {
+	static const char *labels[4];
+	int n = 0;
+
+	labels[n++] = my_yes_label();
+	labels[n++] = my_no_label();
+	if (dialog_vars.help_button)
+	    labels[n++] = my_help_label();
+	labels[n] = 0;
+
+	result = labels;
+    }
+
+    return result;
+}
+
+/*
+ * Map the given button index for dlg_yes_labels() into our exit-code.
+ */
+int
+dlg_yes_buttoncode(int button)
+{
+    int result = DLG_EXIT_ERROR;
+
+    if (dialog_vars.extra_button) {
+	result = dlg_ok_buttoncode(button);
+    } else if (button == 0) {
+	result = DLG_EXIT_OK;
+    } else if (button == 1) {
+	result = DLG_EXIT_CANCEL;
+    } else if (button == 2 && dialog_vars.help_button) {
+	result = DLG_EXIT_HELP;
+    }
+
+    return result;
 }
 
 /*
