@@ -1,29 +1,34 @@
 /*
- *  $Id: checklist.c,v 1.83 2004/12/20 20:42:58 tom Exp $
+ *  $Id: checklist.c,v 1.104 2005/12/20 01:51:38 tom Exp $
  *
  *  checklist.c -- implements the checklist box
  *
- *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
- *     Stuart Herbert - S.Herbert@sheffield.ac.uk: radiolist extension
- *     Alessandro Rubini - rubini@ipvvis.unipv.it: merged the two
- *     Thomas E. Dickey - rewrote...
+ *  Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  An earlier version of this program lists as authors:
+ *	Savio Lam (lam836@cs.cuhk.hk)
+ *	Stuart Herbert - S.Herbert@sheffield.ac.uk: radiolist extension
+ *	Alessandro Rubini - rubini@ipvvis.unipv.it: merged the two
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
 
 static int list_width, check_x, item_x, checkflag;
 
@@ -53,15 +58,20 @@ print_arrows(WINDOW *win,
 }
 
 /*
- * Print list item
+ * Print list item.  The 'selected' parameter is true if 'choice' is the
+ * current item.  That one is colored differently from the other items.
  */
 static void
-print_item(WINDOW *win, char **items, int status,
-	   int choice, int selected)
+print_item(WINDOW *win,
+	   DIALOG_LISTITEM * item,
+	   const char *states,
+	   int choice,
+	   int selected)
 {
     chtype save = getattrs(win);
     int i;
     chtype attr = A_NORMAL;
+    const int *cols;
     const int *indx;
     int limit;
 
@@ -75,72 +85,112 @@ print_item(WINDOW *win, char **items, int status,
     wattrset(win, selected ? check_selected_attr : check_attr);
     (void) wprintw(win,
 		   (checkflag == FLAG_CHECK) ? "[%c]" : "(%c)",
-		   status ? 'X' : ' ');
+		   states[item->state]);
     wattrset(win, menubox_attr);
     (void) waddch(win, ' ');
 
-    if (strlen(ItemName(0)) != 0) {
+    if (strlen(item->name) != 0) {
 
-	indx = dlg_index_wchars(ItemName(0));
-	limit = dlg_count_wchars(ItemName(0));
+	indx = dlg_index_wchars(item->name);
 
 	wattrset(win, selected ? tag_key_selected_attr : tag_key_attr);
-	(void) waddnstr(win, ItemName(0), indx[1]);
+	(void) waddnstr(win, item->name, indx[1]);
 
-	if ((int) strlen(ItemName(0)) > indx[1]) {
-	    limit = dlg_limit_columns(ItemName(0), (item_x - check_x - 6), 1);
+	if ((int) strlen(item->name) > indx[1]) {
+	    limit = dlg_limit_columns(item->name, (item_x - check_x - 6), 1);
 	    if (limit > 1) {
 		wattrset(win, selected ? tag_selected_attr : tag_attr);
 		(void) waddnstr(win,
-				ItemName(0) + indx[1],
+				item->name + indx[1],
 				indx[limit] - indx[1]);
 	    }
 	}
     }
 
-    if (strlen(ItemText(0)) != 0) {
-	indx = dlg_index_wchars(ItemText(0));
-	limit = dlg_limit_columns(ItemText(0), (getmaxx(win) - item_x - 1), 0);
+    if (strlen(item->text) != 0) {
+	cols = dlg_index_columns(item->text);
+	limit = dlg_limit_columns(item->text, (getmaxx(win) - item_x + 1), 0);
 
 	if (limit > 0) {
 	    (void) wmove(win, choice, item_x);
 	    wattrset(win, selected ? item_selected_attr : item_attr);
-	    dlg_print_text(win, ItemText(0), indx[limit], &attr);
+	    dlg_print_text(win, item->text, cols[limit], &attr);
 	}
     }
 
     if (selected) {
-	dlg_item_help(ItemHelp(0));
+	dlg_item_help(item->help);
     }
     wattrset(win, save);
 }
 
 /*
- * Display a dialog box with a list of options that can be turned on or off
- * The `flag' parameter is used to select between radiolist and checklist.
+ * This is an alternate interface to 'checklist' which allows the application
+ * to read the list item states back directly without putting them in the
+ * output buffer.  It also provides for more than two states over which the
+ * check/radio box can display.
  */
 int
-dialog_checklist(const char *title, const char *cprompt, int height, int width,
-		 int list_height, int item_no, char **items, int flag)
+dlg_checklist(const char *title,
+	      const char *cprompt,
+	      int height,
+	      int width,
+	      int list_height,
+	      int item_no,
+	      DIALOG_LISTITEM * items,
+	      const char *states,
+	      int flag,
+	      int *current_item)
 {
-    bool show_status = FALSE;
-    bool separate_output = ((flag == FLAG_CHECK)
-			    && (dialog_vars.separate_output));
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	ENTERKEY_BINDINGS,
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, KEY_RIGHT ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_LEFT ),
+	DLG_KEYS_DATA( DLGK_ITEM_FIRST, KEY_HOME ),
+	DLG_KEYS_DATA( DLGK_ITEM_LAST,	KEY_END ),
+	DLG_KEYS_DATA( DLGK_ITEM_LAST,	KEY_LL ),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,	'+' ),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,	KEY_DOWN ),
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,	'-' ),
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,	KEY_UP ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	KEY_NPAGE ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	DLGK_MOUSE(KEY_NPAGE) ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	KEY_PPAGE ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	DLGK_MOUSE(KEY_PPAGE) ),
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
+#ifdef KEY_RESIZE
+    int old_height = height;
+    int old_width = width;
+#endif
     int i, j, key2, found, x, y, cur_x, cur_y, box_x, box_y;
     int key = 0, fkey;
     int button = dialog_state.visit_items ? -1 : dlg_defaultno_button();
-    int choice = dlg_default_item(items, CHECKBOX_TAGS);
+    int choice = dlg_default_listitem(items);
     int scrollamt = 0;
-    int max_choice, *status;
+    int max_choice;
+    int was_mouse;
     int use_width, name_width, text_width;
     int result = DLG_EXIT_UNKNOWN;
+    int num_states;
     WINDOW *dialog, *list;
     char *prompt = dlg_strclone(cprompt);
     const char **buttons = dlg_ok_labels();
 
+    dlg_does_output();
     dlg_tab_correct_str(prompt);
+
+#ifdef KEY_RESIZE
+  retry:
+#endif
+
     if (list_height == 0) {
-	use_width = dlg_calc_listw(item_no, items, CHECKBOX_TAGS) + 10;
+	use_width = dlg_calc_list_width(item_no, items) + 10;
 	/* calculate height without items (4) */
 	dlg_auto_size(title, prompt, &height, &width, 4, MAX(26, use_width));
 	dlg_calc_listh(&height, &list_height, item_no);
@@ -150,22 +200,21 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
     dlg_print_size(height, width);
     dlg_ctl_size(height, width);
 
+    /* we need at least two states */
+    if (states == 0 || strlen(states) < 2)
+	states = " *";
+    num_states = strlen(states);
+
     checkflag = flag;
 
-    /* Allocate space for storing item on/off status */
-    status = malloc(sizeof(int) * item_no);
-    assert_ptr(status, "dialog_checklist");
-
-    /* Initializes status */
-    for (i = 0; i < item_no; i++)
-	status[i] = !dlg_strcmp(ItemStatus(i), "on");
-
     max_choice = MIN(list_height, item_no);
 
     x = dlg_box_x_ordinate(width);
     y = dlg_box_y_ordinate(height);
 
     dialog = dlg_new_window(height, width, y, x);
+    dlg_register_window(dialog, "checklist", binding);
+    dlg_register_buttons(dialog, "checklist", buttons);
 
     dlg_mouse_setbase(x, y);
 
@@ -195,8 +244,8 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
     name_width = 0;
     /* Find length of longest item to center checklist */
     for (i = 0; i < item_no; i++) {
-	text_width = MAX(text_width, (int) strlen(ItemText(i)));
-	name_width = MAX(name_width, (int) strlen(ItemName(i)));
+	text_width = MAX(text_width, dlg_count_columns(items[i].text));
+	name_width = MAX(name_width, dlg_count_columns(items[i].name));
     }
 
     /* If the name+text is wider than the list is allowed, then truncate
@@ -216,15 +265,17 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
     check_x = (use_width - (text_width + name_width)) / 2;
     item_x = name_width + check_x + 6;
 
-    /* Print the list */
+    /* ensure we are scrolled to show the current choice */
     if (choice >= (max_choice + scrollamt)) {
 	scrollamt = choice - max_choice + 1;
 	choice = max_choice - 1;
     }
+    /* Print the list */
     for (i = 0; i < max_choice; i++)
 	print_item(list,
-		   ItemData(i + scrollamt),
-		   status[i + scrollamt], i, i == choice);
+		   &items[i + scrollamt],
+		   states,
+		   i, i == choice);
     (void) wnoutrefresh(list);
 
     /* register the new window, along with its borders */
@@ -242,20 +293,28 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 	    wmove(dialog, box_y + choice + 1, box_x + check_x + 2);
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
+	if (dlg_result_key(key, fkey, &result))
+	    break;
 
-	if (fkey && (key >= (M_EVENT + KEY_MAX))) {
+	was_mouse = (fkey && is_DLGK_MOUSE(key));
+	if (was_mouse)
+	    key -= M_EVENT;
+
+	if (was_mouse && (key >= KEY_MAX)) {
 	    getyx(dialog, cur_y, cur_x);
-	    i = (key - (M_EVENT + KEY_MAX));
-	    if (scrollamt + i < max_choice) {
+	    i = (key - KEY_MAX);
+	    if (i < max_choice) {
 		/* De-highlight current item */
 		print_item(list,
-			   ItemData(scrollamt + choice),
-			   status[scrollamt + choice], choice, FALSE);
+			   &items[scrollamt + choice],
+			   states,
+			   choice, FALSE);
 		/* Highlight new item */
-		choice = (key - (M_EVENT + KEY_MAX));
+		choice = (key - KEY_MAX);
 		print_item(list,
-			   ItemData(scrollamt + choice),
-			   status[scrollamt + choice], choice, TRUE);
+			   &items[scrollamt + choice],
+			   states,
+			   choice, TRUE);
 		(void) wnoutrefresh(list);
 		(void) wmove(dialog, cur_y, cur_x);
 
@@ -265,6 +324,8 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 		continue;
 	    }
 	    fkey = FALSE;
+	} else if (was_mouse && key >= KEY_MIN) {
+	    key = dlg_lookup_key(dialog, key, &fkey);
 	}
 
 	/*
@@ -273,52 +334,44 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 	 * items can be selected).
 	 */
 	if (key == ' ') {
+	    int current = scrollamt + choice;
+	    int next = items[current].state + 1;
+
+	    if (next >= num_states)
+		next = 0;
+
 	    getyx(dialog, cur_y, cur_x);
 	    if (flag == FLAG_CHECK) {	/* checklist? */
-		status[scrollamt + choice] = !status[scrollamt + choice];
+		items[current].state = next;
 		print_item(list,
-			   ItemData(scrollamt + choice),
-			   status[scrollamt + choice], choice, TRUE);
+			   &items[scrollamt + choice],
+			   states,
+			   choice, TRUE);
 	    } else {		/* radiolist */
-		if (!status[scrollamt + choice]) {
-		    for (i = 0; i < item_no; i++)
-			status[i] = FALSE;
-		    status[scrollamt + choice] = TRUE;
+		for (i = 0; i < item_no; i++) {
+		    if (i != current) {
+			items[i].state = 0;
+		    }
+		}
+		if (items[current].state) {
+		    items[current].state = next ? next : 1;
+		    print_item(list,
+			       &items[current],
+			       states,
+			       choice, TRUE);
+		} else {
+		    items[current].state = 1;
 		    for (i = 0; i < max_choice; i++)
 			print_item(list,
-				   ItemData(scrollamt + i),
-				   status[scrollamt + i], i, i == choice);
+				   &items[scrollamt + i],
+				   states,
+				   i, i == choice);
 		}
 	    }
 	    (void) wnoutrefresh(list);
 	    (void) wmove(dialog, cur_y, cur_x);
 	    wrefresh(dialog);
 	    continue;		/* wait for another key press */
-	} else if (key == ESC) {
-	    result = DLG_EXIT_ESC;
-	    continue;
-	}
-
-	if (!fkey) {
-	    fkey = TRUE;
-	    switch (key) {
-	    case '\n':
-	    case '\r':
-		key = KEY_ENTER;
-		break;
-	    case '-':
-		key = KEY_UP;
-		break;
-	    case '+':
-		key = KEY_DOWN;
-		break;
-	    case TAB:
-		key = KEY_RIGHT;
-		break;
-	    default:
-		fkey = FALSE;
-		break;
-	    }
 	}
 
 	/*
@@ -330,7 +383,7 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 	if (!fkey) {
 	    if (button < 0 || !dialog_state.visit_items) {
 		for (j = scrollamt + choice + 1; j < item_no; j++) {
-		    if (dlg_match_char(dlg_last_getc(), ItemName(j))) {
+		    if (dlg_match_char(dlg_last_getc(), items[j].name)) {
 			found = TRUE;
 			i = j - scrollamt;
 			break;
@@ -338,7 +391,7 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 		}
 		if (!found) {
 		    for (j = 0; j <= scrollamt + choice; j++) {
-			if (dlg_match_char(dlg_last_getc(), ItemName(j))) {
+			if (dlg_match_char(dlg_last_getc(), items[j].name)) {
 			    found = TRUE;
 			    i = j - scrollamt;
 			    break;
@@ -370,15 +423,13 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 	    if (fkey) {
 		found = TRUE;
 		switch (key) {
-		case KEY_HOME:
+		case DLGK_ITEM_FIRST:
 		    i = -scrollamt;
 		    break;
-		case KEY_LL:
-		case KEY_END:
+		case DLGK_ITEM_LAST:
 		    i = item_no - 1 - scrollamt;
 		    break;
-		case M_EVENT + KEY_PPAGE:
-		case KEY_PPAGE:
+		case DLGK_PAGE_PREV:
 		    if (choice)
 			i = 0;
 		    else if (scrollamt != 0)
@@ -386,16 +437,15 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 		    else
 			continue;
 		    break;
-		case M_EVENT + KEY_NPAGE:
-		case KEY_NPAGE:
+		case DLGK_PAGE_NEXT:
 		    i = MIN(choice + max_choice, item_no - scrollamt - 1);
 		    break;
-		case KEY_UP:
+		case DLGK_ITEM_PREV:
 		    i = choice - 1;
 		    if (choice == 0 && scrollamt == 0)
 			continue;
 		    break;
-		case KEY_DOWN:
+		case DLGK_ITEM_NEXT:
 		    i = choice + 1;
 		    if (scrollamt + choice >= item_no - 1)
 			continue;
@@ -420,22 +470,24 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 			if (list_height > 1) {
 			    /* De-highlight current first item */
 			    print_item(list,
-				       ItemData(scrollamt),
-				       status[scrollamt], 0, FALSE);
+				       &items[scrollamt],
+				       states,
+				       0, FALSE);
 			    scrollok(list, TRUE);
 			    wscrl(list, -1);
 			    scrollok(list, FALSE);
 			}
 			scrollamt--;
 			print_item(list,
-				   ItemData(scrollamt),
-				   status[scrollamt], 0, TRUE);
+				   &items[scrollamt],
+				   states,
+				   0, TRUE);
 		    } else if (i == max_choice) {
 			if (list_height > 1) {
 			    /* De-highlight current last item before scrolling up */
 			    print_item(list,
-				       ItemData(scrollamt + max_choice - 1),
-				       status[scrollamt + max_choice - 1],
+				       &items[scrollamt + max_choice - 1],
+				       states,
 				       max_choice - 1, FALSE);
 			    scrollok(list, TRUE);
 			    wscrl(list, 1);
@@ -443,8 +495,8 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 			}
 			scrollamt++;
 			print_item(list,
-				   ItemData(scrollamt + max_choice - 1),
-				   status[scrollamt + max_choice - 1],
+				   &items[scrollamt + max_choice - 1],
+				   states,
 				   max_choice - 1, TRUE);
 		    } else
 #endif
@@ -458,8 +510,9 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 			}
 			for (i = 0; i < max_choice; i++) {
 			    print_item(list,
-				       ItemData(scrollamt + i),
-				       status[scrollamt + i], i, i == choice);
+				       &items[scrollamt + i],
+				       states,
+				       i, i == choice);
 			}
 		    }
 		    (void) wnoutrefresh(list);
@@ -469,13 +522,15 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 		} else {
 		    /* De-highlight current item */
 		    print_item(list,
-			       ItemData(scrollamt + choice),
-			       status[scrollamt + choice], choice, FALSE);
+			       &items[scrollamt + choice],
+			       states,
+			       choice, FALSE);
 		    /* Highlight new item */
 		    choice = i;
 		    print_item(list,
-			       ItemData(scrollamt + choice),
-			       status[scrollamt + choice], choice, TRUE);
+			       &items[scrollamt + choice],
+			       states,
+			       choice, TRUE);
 		    (void) wnoutrefresh(list);
 		    (void) wmove(dialog, cur_y, cur_x);
 		    wrefresh(dialog);
@@ -486,23 +541,35 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 
 	if (fkey) {
 	    switch (key) {
-	    case KEY_ENTER:
+	    case DLGK_ENTER:
 		result = dlg_ok_buttoncode(button);
 		break;
-	    case KEY_BTAB:
-	    case KEY_LEFT:
+	    case DLGK_FIELD_PREV:
 		button = dlg_prev_button(buttons, button);
 		dlg_draw_buttons(dialog, height - 2, 0, buttons, button,
 				 FALSE, width);
 		break;
-	    case KEY_RIGHT:
+	    case DLGK_FIELD_NEXT:
 		button = dlg_next_button(buttons, button);
 		dlg_draw_buttons(dialog, height - 2, 0, buttons, button,
 				 FALSE, width);
 		break;
+#ifdef KEY_RESIZE
+	    case KEY_RESIZE:
+		/* reset data */
+		height = old_height;
+		width = old_width;
+		/* repaint */
+		dlg_clear();
+		dlg_del_window(dialog);
+		refresh();
+		dlg_mouse_free_regions();
+		goto retry;
+		break;
+#endif
 	    default:
-		if (key >= M_EVENT) {
-		    if ((key2 = dlg_ok_buttoncode(key - M_EVENT)) >= 0) {
+		if (was_mouse) {
+		    if ((key2 = dlg_ok_buttoncode(key)) >= 0) {
 			result = key2;
 			break;
 		    }
@@ -515,6 +582,55 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
     }
 
     dlg_del_window(dialog);
+    dlg_mouse_free_regions();
+    free(prompt);
+    *current_item = (scrollamt + choice);
+    return result;
+}
+
+/*
+ * Display a dialog box with a list of options that can be turned on or off
+ * The `flag' parameter is used to select between radiolist and checklist.
+ */
+int
+dialog_checklist(const char *title,
+		 const char *cprompt,
+		 int height,
+		 int width,
+		 int list_height,
+		 int item_no,
+		 char **items,
+		 int flag)
+{
+    int result;
+    int i;
+    DIALOG_LISTITEM *listitems;
+    bool separate_output = ((flag == FLAG_CHECK)
+			    && (dialog_vars.separate_output));
+    bool show_status = FALSE;
+    int current = 0;
+
+    listitems = calloc(item_no + 1, sizeof(*listitems));
+    assert_ptr(listitems, "dialog_checklist");
+
+    for (i = 0; i < item_no; ++i) {
+	listitems[i].name = ItemName(i);
+	listitems[i].text = ItemText(i);
+	listitems[i].help = (dialog_vars.item_help) ? ItemHelp(i) : "";
+	listitems[i].state = !dlg_strcmp(ItemStatus(i), "on");
+    }
+
+    result = dlg_checklist(title,
+			   cprompt,
+			   height,
+			   width,
+			   list_height,
+			   item_no,
+			   listitems,
+			   NULL,
+			   flag,
+			   &current);
+
     switch (result) {
     case DLG_EXIT_OK:		/* FALLTHRU */
     case DLG_EXIT_EXTRA:
@@ -523,28 +639,28 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
     case DLG_EXIT_HELP:
 	dlg_add_result("HELP ");
 	show_status = dialog_vars.help_status;
-	if (USE_ITEM_HELP(ItemHelp(scrollamt + choice))) {
+	if (USE_ITEM_HELP(listitems[current].help)) {
 	    if (show_status) {
 		if (separate_output) {
-		    dlg_add_result(ItemHelp(scrollamt + choice));
+		    dlg_add_result(listitems[current].help);
 		    dlg_add_result("\n");
 		} else {
-		    dlg_add_quoted(ItemHelp(scrollamt + choice));
+		    dlg_add_quoted(listitems[current].help);
 		}
 	    } else {
-		dlg_add_result(ItemHelp(scrollamt + choice));
+		dlg_add_result(listitems[current].help);
 	    }
 	    result = DLG_EXIT_ITEM_HELP;
 	} else {
 	    if (show_status) {
 		if (separate_output) {
-		    dlg_add_result(ItemName(scrollamt + choice));
+		    dlg_add_result(listitems[current].name);
 		    dlg_add_result("\n");
 		} else {
-		    dlg_add_quoted(ItemName(scrollamt + choice));
+		    dlg_add_quoted(listitems[current].name);
 		}
 	    } else {
-		dlg_add_result(ItemName(scrollamt + choice));
+		dlg_add_result(listitems[current].name);
 	    }
 	}
 	break;
@@ -552,24 +668,23 @@ dialog_checklist(const char *title, const char *cprompt, int height, int width,
 
     if (show_status) {
 	for (i = 0; i < item_no; i++) {
-	    if (status[i]) {
+	    if (listitems[i].state) {
 		if (separate_output) {
-		    dlg_add_result(ItemName(i));
+		    dlg_add_result(listitems[i].name);
 		    dlg_add_result("\n");
 		} else {
-		    if (*(dialog_vars.input_result))
+		    if (dialog_vars.input_result && *(dialog_vars.input_result))
 			dlg_add_result(" ");
 		    if (flag == FLAG_CHECK) {
-			dlg_add_quoted(ItemName(i));
+			dlg_add_quoted(listitems[i].name);
 		    } else {
-			dlg_add_result(ItemName(i));
+			dlg_add_result(listitems[i].name);
 		    }
 		}
 	    }
 	}
     }
-    dlg_mouse_free_regions();
-    free(status);
-    free(prompt);
+
+    free(listitems);
     return result;
 }
