@@ -1,27 +1,31 @@
 /*
- * $Id: dialog.c,v 1.130 2005/01/16 16:53:48 tom Exp $
+ * $Id: dialog.c,v 1.139 2005/12/08 01:03:55 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
- *     and: Thomas E. Dickey
+ *  Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  An earlier version of this program lists as authors
+ *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include "dialog.h"
+#include <dialog.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -30,8 +34,9 @@
 #include <locale.h>
 #endif
 
-#define JUMPARGS const char *t, char *av[], int *offset_add
-typedef int (jumperFn) (JUMPARGS);
+#define PASSARGS             t,       av,        offset_add
+#define CALLARGS const char *t, char *av[], int *offset_add
+typedef int (callerFn) (CALLARGS);
 
 typedef enum {
     o_unknown = 0
@@ -88,6 +93,7 @@ typedef enum {
     ,o_ok_label
     ,o_output_fd
     ,o_passwordbox
+    ,o_passwordform
     ,o_pause
     ,o_print_maxsize
     ,o_print_size
@@ -137,7 +143,7 @@ typedef struct {
 typedef struct {
     eOptions code;
     int argmin, argmax;
-    jumperFn *jumper;
+    callerFn *jumper;
 } Mode;
 
 static bool *dialog_opts;
@@ -209,6 +215,7 @@ static const Options options[] = {
     { "ok-label",	o_ok_label,		1, "<str>" },
     { "output-fd",	o_output_fd,		1, "<fd>" },
     { "passwordbox",	o_passwordbox,		2, "<text> <height> <width> [<init>]" },
+    { "passwordform",	o_passwordform,     	2, "<text> <height> <width> <form height> <label1> <l_y1> <l_x1> <item1> <i_y1> <i_x1> <flen1> <ilen1>..." },
     { "pause",		o_pause,		2, "<text> <height> <width> <seconds>" },
     { "print-maxsize",	o_print_maxsize,	1, "" },
     { "print-size",	o_print_size,		1, "" },
@@ -596,8 +603,11 @@ show_result(int ret)
     case DLG_EXIT_EXTRA:
     case DLG_EXIT_HELP:
     case DLG_EXIT_ITEM_HELP:
-	if (dialog_state.output_count > 1) {
-	    fputs(dialog_state.separate_str, dialog_state.output);
+	if ((dialog_state.output_count > 1) && !dialog_vars.separate_output) {
+	    fputs((dialog_state.separate_str
+		   ? dialog_state.separate_str
+		   : DEFAULT_SEPARATE_STR),
+		  dialog_state.output);
 	    either = TRUE;
 	}
 	if (dialog_vars.input_result[0] != '\0') {
@@ -613,11 +623,11 @@ show_result(int ret)
 }
 
 /*
- * These are the program jumpers
+ * These are the widget callers.
  */
 
 static int
-j_yesno(JUMPARGS)
+call_yesno(CALLARGS)
 {
     *offset_add = 4;
     return dialog_yesno(t,
@@ -627,7 +637,7 @@ j_yesno(JUMPARGS)
 }
 
 static int
-j_msgbox(JUMPARGS)
+call_msgbox(CALLARGS)
 {
     *offset_add = 4;
     return dialog_msgbox(t,
@@ -637,7 +647,7 @@ j_msgbox(JUMPARGS)
 }
 
 static int
-j_infobox(JUMPARGS)
+call_infobox(CALLARGS)
 {
     *offset_add = 4;
     return dialog_msgbox(t,
@@ -647,7 +657,7 @@ j_infobox(JUMPARGS)
 }
 
 static int
-j_textbox(JUMPARGS)
+call_textbox(CALLARGS)
 {
     *offset_add = 4;
     return dialog_textbox(t,
@@ -657,7 +667,7 @@ j_textbox(JUMPARGS)
 }
 
 static int
-j_menu(JUMPARGS)
+call_menu(CALLARGS)
 {
     int tags = howmany_tags(av + 5, MENUBOX_TAGS);
     *offset_add = 5 + tags * MENUBOX_TAGS;
@@ -671,7 +681,7 @@ j_menu(JUMPARGS)
 }
 
 static int
-j_inputmenu(JUMPARGS)
+call_inputmenu(CALLARGS)
 {
     int tags = howmany_tags(av + 5, MENUBOX_TAGS);
 
@@ -695,7 +705,7 @@ j_inputmenu(JUMPARGS)
 }
 
 static int
-j_checklist(JUMPARGS)
+call_checklist(CALLARGS)
 {
     int tags = howmany_tags(av + 5, CHECKBOX_TAGS);
     *offset_add = 5 + tags * CHECKBOX_TAGS;
@@ -708,7 +718,7 @@ j_checklist(JUMPARGS)
 }
 
 static int
-j_radiolist(JUMPARGS)
+call_radiolist(CALLARGS)
 {
     int tags = howmany_tags(av + 5, CHECKBOX_TAGS);
     *offset_add = 5 + tags * CHECKBOX_TAGS;
@@ -721,7 +731,7 @@ j_radiolist(JUMPARGS)
 }
 
 static int
-j_inputbox(JUMPARGS)
+call_inputbox(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_inputbox(t,
@@ -732,7 +742,7 @@ j_inputbox(JUMPARGS)
 }
 
 static int
-j_passwordbox(JUMPARGS)
+call_passwordbox(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_inputbox(t,
@@ -744,7 +754,7 @@ j_passwordbox(JUMPARGS)
 
 #ifdef HAVE_XDIALOG
 static int
-j_calendar(JUMPARGS)
+call_calendar(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_calendar(t,
@@ -757,7 +767,7 @@ j_calendar(JUMPARGS)
 }
 
 static int
-j_fselect(JUMPARGS)
+call_fselect(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_fselect(t,
@@ -767,7 +777,7 @@ j_fselect(JUMPARGS)
 }
 
 static int
-j_timebox(JUMPARGS)
+call_timebox(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_timebox(t,
@@ -782,7 +792,7 @@ j_timebox(JUMPARGS)
 
 #ifdef HAVE_FORMBOX
 static int
-j_form(JUMPARGS)
+call_form(CALLARGS)
 {
     int tags = howmany_tags(av + 5, FORMBOX_TAGS);
     *offset_add = 5 + tags * FORMBOX_TAGS;
@@ -794,11 +804,24 @@ j_form(JUMPARGS)
 		       numeric_arg(av, 4),
 		       tags, av + 5);
 }
+
+static int
+call_password_form(CALLARGS)
+{
+    int save = dialog_vars.formitem_type;
+    int result;
+
+    dialog_vars.formitem_type = 1;
+    result = call_form(PASSARGS);
+    dialog_vars.formitem_type = save;
+
+    return result;
+}
 #endif
 
 #ifdef HAVE_GAUGE
 static int
-j_gauge(JUMPARGS)
+call_gauge(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_gauge(t,
@@ -810,7 +833,7 @@ j_gauge(JUMPARGS)
 #endif
 
 static int
-j_pause(JUMPARGS)
+call_pause(CALLARGS)
 {
     *offset_add = arg_rest(av);
     return dialog_pause(t,
@@ -822,7 +845,7 @@ j_pause(JUMPARGS)
 
 #ifdef HAVE_TAILBOX
 static int
-j_tailbox(JUMPARGS)
+call_tailbox(CALLARGS)
 {
     *offset_add = 4;
     return dialog_tailbox(t,
@@ -833,7 +856,7 @@ j_tailbox(JUMPARGS)
 }
 
 static int
-j_tailboxbg(JUMPARGS)
+call_tailboxbg(CALLARGS)
 {
     *offset_add = 4;
     return dialog_tailbox(t,
@@ -846,31 +869,32 @@ j_tailboxbg(JUMPARGS)
 /* *INDENT-OFF* */
 static const Mode modes[] =
 {
-    {o_yesno,       4, 4, j_yesno},
-    {o_msgbox,      4, 4, j_msgbox},
-    {o_infobox,     4, 4, j_infobox},
-    {o_textbox,     4, 4, j_textbox},
-    {o_menu,        7, 0, j_menu},
-    {o_inputmenu,   7, 0, j_inputmenu},
-    {o_checklist,   8, 0, j_checklist},
-    {o_radiolist,   8, 0, j_radiolist},
-    {o_inputbox,    4, 5, j_inputbox},
-    {o_passwordbox, 4, 5, j_passwordbox},
-    {o_pause,       5, 5, j_pause},
+    {o_yesno,           4, 4, call_yesno},
+    {o_msgbox,          4, 4, call_msgbox},
+    {o_infobox,         4, 4, call_infobox},
+    {o_textbox,         4, 4, call_textbox},
+    {o_menu,            7, 0, call_menu},
+    {o_inputmenu,       7, 0, call_inputmenu},
+    {o_checklist,       8, 0, call_checklist},
+    {o_radiolist,       8, 0, call_radiolist},
+    {o_inputbox,        4, 5, call_inputbox},
+    {o_passwordbox,     4, 5, call_passwordbox},
+    {o_pause,           5, 5, call_pause},
 #ifdef HAVE_XDIALOG
-    {o_calendar,    4, 7, j_calendar},
-    {o_fselect,     4, 5, j_fselect},
-    {o_timebox,     4, 7, j_timebox},
+    {o_calendar,        4, 7, call_calendar},
+    {o_fselect,         4, 5, call_fselect},
+    {o_timebox,         4, 7, call_timebox},
 #endif
 #ifdef HAVE_FORMBOX
-    {o_form,       13, 0, j_form},
+    {o_passwordform,   13, 0, call_password_form},
+    {o_form,           13, 0, call_form},
 #endif
 #ifdef HAVE_GAUGE
-    {o_gauge,       4, 5, j_gauge},
+    {o_gauge,           4, 5, call_gauge},
 #endif
 #ifdef HAVE_TAILBOX
-    {o_tailbox,     4, 4, j_tailbox},
-    {o_tailboxbg,   4, 4, j_tailboxbg},
+    {o_tailbox,         4, 4, call_tailbox},
+    {o_tailboxbg,       4, 4, call_tailboxbg},
 #endif
 };
 /* *INDENT-ON* */
@@ -958,7 +982,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright (C) 2004 Thomas E. Dickey",
+	"Copyright (C) 2005 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1233,8 +1257,14 @@ init_result(char *buffer)
 	    special_argc = count_argv(special_argv);
 	}
     }
-    if (special_argv != 0)
+    if (special_argv != 0) {
 	process_common_options(special_argc, special_argv, 0, FALSE);
+#ifdef NO_LEAKS
+	free(special_argv[0]);
+	free(special_argv);
+	first = TRUE;
+#endif
+    }
 }
 
 int
