@@ -1,27 +1,32 @@
 /*
- *  $Id: formbox.c,v 1.32 2005/02/07 00:10:37 tom Exp $
+ *  $Id: formbox.c,v 1.51 2005/12/20 00:25:59 tom Exp $
  *
  *  formbox.c -- implements the form (i.e, some pairs label/editbox)
  *
- *  AUTHOR: Valery Reznic (valery_reznic@users.sourceforge.net)
- *     and: Thomas E. Dickey
+ *  Copyright 2003-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  This is adapted from source contributed by
+ *	Valery Reznic (valery_reznic@users.sourceforge.net)
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
 
 #define LLEN(n) ((n) * FORMBOX_TAGS)
 
@@ -35,19 +40,7 @@
 #define ItemTextILen(i) items[LLEN(i) + 7]
 #define ItemHelp(i)     (dialog_vars.item_help ? items[LLEN(i) + 8] : "")
 
-typedef struct {
-    char *name;			/* the field label */
-    int name_len;		/* ...its length */
-    int name_y;			/* ...its y-ordinate */
-    int name_x;			/* ...its x-ordinate */
-    char *text;			/* the field contents */
-    int text_len;		/* ...its length on the screen */
-    int text_y;			/* ...its y-ordinate */
-    int text_x;			/* ...its x-ordinate */
-    int text_flen;		/* ...its length on screen, or 0 if no input allowed */
-    int text_ilen;		/* ...its limit on amount to be entered */
-    char *help;			/* help-message, if any */
-} FORM_ELT;
+#define isPassword(item) ((item)->type == 1)
 
 static bool
 in_window(WINDOW *win, int scrollamt, int y)
@@ -73,42 +66,42 @@ move_past(WINDOW *win, int y, int x)
  * Print form item
  */
 static int
-print_item(WINDOW *win, FORM_ELT * elt, int scrollamt, bool choice)
+print_item(WINDOW *win, DIALOG_FORMITEM * item, int scrollamt, bool choice)
 {
     int count = 0;
     int len;
 
-    if (ok_move(win, scrollamt, elt->name_y, elt->name_x)) {
-	len = elt->name_len;
-	len = MIN(len, getmaxx(win) - elt->name_x);
+    if (ok_move(win, scrollamt, item->name_y, item->name_x)) {
+	len = item->name_len;
+	len = MIN(len, getmaxx(win) - item->name_x);
 	if (len > 0) {
 	    dlg_show_string(win,
-			    elt->name,
+			    item->name,
 			    0,
 			    menubox_attr,
-			    elt->name_y,
-			    elt->name_x,
+			    item->name_y - scrollamt,
+			    item->name_x,
 			    len,
 			    FALSE,
 			    FALSE);
-	    move_past(win, elt->name_y, elt->name_x + len);
+	    move_past(win, item->name_y - scrollamt, item->name_x + len);
 	    count = 1;
 	}
     }
-    if (elt->text_len && ok_move(win, scrollamt, elt->text_y, elt->text_x)) {
-	len = elt->text_len;
-	len = MIN(len, getmaxx(win) - elt->text_x);
+    if (item->text_len && ok_move(win, scrollamt, item->text_y, item->text_x)) {
+	len = item->text_len;
+	len = MIN(len, getmaxx(win) - item->text_x);
 	if (len > 0) {
 	    dlg_show_string(win,
-			    elt->text,
+			    item->text,
 			    0,
 			    choice ? form_active_text_attr : form_text_attr,
-			    elt->text_y,
-			    elt->text_x,
+			    item->text_y - scrollamt,
+			    item->text_x,
 			    len,
-			    FALSE,
+			    isPassword(item),
 			    FALSE);
-	    move_past(win, elt->text_y, elt->text_x + len);
+	    move_past(win, item->text_y - scrollamt, item->text_x + len);
 	    count = 1;
 	}
     }
@@ -119,13 +112,13 @@ print_item(WINDOW *win, FORM_ELT * elt, int scrollamt, bool choice)
  * Print the entire form.
  */
 static void
-print_form(WINDOW *win, FORM_ELT * elt, int total, int scrollamt, int choice)
+print_form(WINDOW *win, DIALOG_FORMITEM * item, int total, int scrollamt, int choice)
 {
     int n;
     int count = 0;
 
     for (n = 0; n < total; ++n) {
-	count += print_item(win, elt + n, scrollamt, n == choice);
+	count += print_item(win, item + n, scrollamt, n == choice);
     }
     if (count) {
 	wbkgdset(win, menubox_attr | ' ');
@@ -135,13 +128,13 @@ print_form(WINDOW *win, FORM_ELT * elt, int total, int scrollamt, int choice)
 }
 
 static int
-set_choice(FORM_ELT elt[], int choice, int item_no)
+set_choice(DIALOG_FORMITEM item[], int choice, int item_no)
 {
     int i;
-    if (elt[choice].text_flen > 0)
+    if (item[choice].text_flen > 0)
 	return choice;
     for (i = 0; i < item_no; i++) {
-	if (elt[i].text_flen > 0)
+	if (item[i].text_flen > 0)
 	    return i;
     }
     dlg_exiterr("No field has flen > 0\n");
@@ -152,15 +145,15 @@ set_choice(FORM_ELT elt[], int choice, int item_no)
  * Find the last y-value in the form.
  */
 static int
-form_limit(FORM_ELT elt[])
+form_limit(DIALOG_FORMITEM item[])
 {
     int n;
     int limit = 0;
-    for (n = 0; elt[n].name != 0; ++n) {
-	if (limit < elt[n].name_y)
-	    limit = elt[n].name_y;
-	if (limit < elt[n].text_y)
-	    limit = elt[n].text_y;
+    for (n = 0; item[n].name != 0; ++n) {
+	if (limit < item[n].name_y)
+	    limit = item[n].name_y;
+	if (limit < item[n].text_y)
+	    limit = item[n].text_y;
     }
     return limit;
 }
@@ -170,7 +163,7 @@ form_limit(FORM_ELT elt[])
  */
 static bool
 tab_next(WINDOW *win,
-	 FORM_ELT elt[],
+	 DIALOG_FORMITEM item[],
 	 int item_no,
 	 int stepsize,
 	 int *choice,
@@ -190,13 +183,13 @@ tab_next(WINDOW *win,
 	    *choice = 0;
 	    wrapped = TRUE;
 	}
-	if (elt[*choice].text_flen > 0) {
-	    int lo = MIN(elt[*choice].name_y, elt[*choice].text_y);
-	    int hi = MAX(elt[*choice].name_y, elt[*choice].text_y);
+	if (item[*choice].text_flen > 0) {
+	    int lo = MIN(item[*choice].name_y, item[*choice].text_y);
+	    int hi = MAX(item[*choice].name_y, item[*choice].text_y);
 
 	    if (old_choice == *choice)
 		break;
-	    print_item(win, elt + old_choice, *scrollamt, FALSE);
+	    print_item(win, item + old_choice, *scrollamt, FALSE);
 
 	    if (*scrollamt < lo + 1 - getmaxy(win))
 		*scrollamt = lo + 1 - getmaxy(win);
@@ -232,11 +225,11 @@ tab_next(WINDOW *win,
  * nor is there necessarily a field on each row of the window.
  */
 static bool
-scroll_next(WINDOW *win, FORM_ELT elt[], int stepsize, int *choice, int *scrollamt)
+scroll_next(WINDOW *win, DIALOG_FORMITEM item[], int stepsize, int *choice, int *scrollamt)
 {
     int old_choice = *choice;
     int old_scroll = *scrollamt;
-    int old_row = MIN(elt[old_choice].text_y, elt[old_choice].name_y);
+    int old_row = MIN(item[old_choice].text_y, item[old_choice].name_y);
     int target = old_scroll + stepsize;
     int n;
 
@@ -248,14 +241,14 @@ scroll_next(WINDOW *win, FORM_ELT elt[], int stepsize, int *choice, int *scrolla
 	if (target < 0)
 	    target = 0;
     } else {
-	int limit = form_limit(elt);
+	int limit = form_limit(item);
 	if (target > limit)
 	    target = limit;
     }
 
-    for (n = 0; elt[n].name != 0; ++n) {
-	if (elt[n].text_flen > 0) {
-	    int new_row = MIN(elt[n].text_y, elt[n].name_y);
+    for (n = 0; item[n].name != 0; ++n) {
+	if (item[n].text_flen > 0) {
+	    int new_row = MIN(item[n].text_y, item[n].name_y);
 	    if (abs(new_row - target) < abs(old_row - target)) {
 		old_row = new_row;
 		*choice = n;
@@ -264,7 +257,7 @@ scroll_next(WINDOW *win, FORM_ELT elt[], int stepsize, int *choice, int *scrolla
     }
 
     if (old_choice != *choice)
-	print_item(win, elt + old_choice, *scrollamt, FALSE);
+	print_item(win, item + old_choice, *scrollamt, FALSE);
 
     *scrollamt = *choice;
     if (*scrollamt != old_scroll) {
@@ -275,96 +268,143 @@ scroll_next(WINDOW *win, FORM_ELT elt[], int stepsize, int *choice, int *scrolla
     return (old_choice != *choice) || (old_scroll != *scrollamt);
 }
 
-static FORM_ELT *
-init_fe(char **items,
-	int item_no,
-	int *min_height,
-	int *min_width)
+/*
+ * Do a sanity check on the field length, and return the "right" value.
+ */
+static int
+real_length(DIALOG_FORMITEM * item)
+{
+    return (item->text_flen > 0
+	    ? item->text_flen
+	    : (item->text_flen < 0
+	       ? -item->text_flen
+	       : item->text_len));
+}
+
+/*
+ * Compute the form size, setup field buffers.
+ */
+static void
+make_FORM_ELTs(DIALOG_FORMITEM * item,
+	       int item_no,
+	       int *min_height,
+	       int *min_width)
 {
-    FORM_ELT *elt;
     int i;
     int min_w = 0;
     int min_h = 0;
 
-    /*
-     * Note that elt[item_no].name is null, since we allocate an extra item.
-     */
-    elt = (FORM_ELT *) calloc(item_no + 1, sizeof(FORM_ELT));
-    assert_ptr(elt, "dialog_form");
-
     for (i = 0; i < item_no; ++i) {
-	int name_y = atoi(ItemNameY(i));
-	int name_x = atoi(ItemNameX(i));
-	int name_len = strlen(ItemName(i));
-	int text_y = atoi(ItemTextY(i));
-	int text_x = atoi(ItemTextX(i));
-	int text_flen = atoi(ItemTextFLen(i));
-	int text_ilen = atoi(ItemTextILen(i));
-	int text_len = (text_flen > 0
-			? text_flen
-			: (text_flen < 0
-			   ? -text_flen
-			   : (int) strlen(ItemText(i))));
+	int real_len = real_length(item + i);
 
 	/*
 	 * Special value '0' for text_flen: no input allowed
-	 * Special value '0' for text_ilen: 'be as text_flen'
+	 * Special value '0' for text_ilen: 'be the same as text_flen'
 	 */
-	if (text_ilen == 0)
-	    text_ilen = text_len;
+	if (item[i].text_ilen == 0)
+	    item[i].text_ilen = real_len;
 
-	min_h = MAX(min_h, name_y);
-	min_h = MAX(min_h, text_y);
-	min_w = MAX(min_w, name_x + name_len);
-	min_w = MAX(min_w, text_x + text_len);
+	min_h = MAX(min_h, item[i].name_y + 1);
+	min_h = MAX(min_h, item[i].text_y + 1);
+	min_w = MAX(min_w, item[i].name_x + 1 + item[i].name_len);
+	min_w = MAX(min_w, item[i].text_x + 1 + real_len);
 
-	elt[i].name = ItemName(i);
-	elt[i].name_len = strlen(elt[i].name);
-	elt[i].name_y = name_y - 1;
-	elt[i].name_x = name_x - 1;
-	elt[i].text = ItemText(i);
-	elt[i].text_len = text_len;
-	elt[i].text_y = text_y - 1;
-	elt[i].text_x = text_x - 1;
-	elt[i].text_flen = text_flen;
-	elt[i].text_ilen = text_ilen;
-	elt[i].help = ItemHelp(i);
+	item[i].text_len = real_length(item + i);
+
+	/*
+	 * We do not know the actual length of .text, so we allocate it here
+	 * to ensure it is big enough.
+	 */
+	if (item[i].text_flen > 0) {
+	    int max_len = MAX(item[i].text_ilen + 1, MAX_LEN);
+	    char *old_text = item[i].text;
 
-	if (text_flen > 0) {
-	    int max_len = MAX_LEN;
 	    if (dialog_vars.max_input != 0 && dialog_vars.max_input < MAX_LEN)
 		max_len = dialog_vars.max_input;
-	    elt[i].text = malloc(max_len + 1);
-	    assert_ptr(elt[i].text, "dialog_form");
-	    sprintf(elt[i].text, "%.*s", text_ilen, ItemText(i));
+
+	    item[i].text = malloc(max_len + 1);
+	    assert_ptr(item[i].text, "dialog_form");
+
+	    sprintf(item[i].text, "%.*s", item[i].text_ilen, old_text);
+
+	    if (item[i].text_free) {
+		item[i].text_free = FALSE;
+		free(old_text);
+	    }
+	    item[i].text_free = TRUE;
 	}
     }
 
     *min_height = min_h;
     *min_width = min_w;
+}
 
-    return elt;
+int
+dlg_default_formitem(DIALOG_FORMITEM * items)
+{
+    int result = 0;
+
+    if (dialog_vars.default_item != 0) {
+	int count = 0;
+	while (items->name != 0) {
+	    if (!strcmp(dialog_vars.default_item, items->name)) {
+		result = count;
+		break;
+	    }
+	    ++items;
+	    count++;
+	}
+    }
+    return result;
 }
 
 /*
  * Display a form for fulfill a number of fields
  */
 int
-dialog_form(const char *title, const char *cprompt, int height, int width,
-	    int form_height, int item_no, char **items)
+dlg_form(const char *title,
+	 const char *cprompt,
+	 int height,
+	 int width,
+	 int form_height,
+	 int item_no,
+	 DIALOG_FORMITEM * items,
+	 int *current_item)
 {
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	INPUTSTR_BINDINGS,
+	ENTERKEY_BINDINGS,
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  CHR_NEXT ),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_DOWN ),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_NEXT ),
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,  CHR_PREVIOUS ),
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,  KEY_PREVIOUS ),
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,  KEY_UP ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  KEY_NPAGE ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,  KEY_PPAGE ),
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
+#ifdef KEY_RESIZE
+    int old_height = height;
+    int old_width = width;
+#endif
+
 #define sTEXT -1
-    bool show_status = FALSE;
+
     int form_width;
     int first = TRUE;
     int chr_offset = 0;
     int state = dialog_vars.defaultno ? dlg_defaultno_button() : sTEXT;
-    const int password = 0;
-    int i, x, y, cur_x, cur_y, box_x, box_y;
+    int x, y, cur_x, cur_y, box_x, box_y;
     int code;
     int key = 0;
     int fkey;
-    int choice = dlg_default_item(items, FORMBOX_TAGS);
+    int choice = dlg_default_formitem(items);
     int new_choice, new_scroll;
     int scrollamt = 0;
     int result = DLG_EXIT_UNKNOWN;
@@ -376,10 +416,16 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
     WINDOW *dialog, *form;
     char *prompt = dlg_strclone(cprompt);
     const char **buttons = dlg_ok_labels();
-    FORM_ELT *elt, *current;
+    DIALOG_FORMITEM *current;
 
-    elt = init_fe(items, item_no, &min_height, &min_width);
+    make_FORM_ELTs(items, item_no, &min_height, &min_width);
+    dlg_does_output();
     dlg_tab_correct_str(prompt);
+
+#ifdef KEY_RESIZE
+  retry:
+#endif
+
     dlg_auto_size(title, prompt, &height, &width,
 		  1 + 3 * MARGIN,
 		  MAX(26, 2 + min_width));
@@ -405,6 +451,8 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
     y = dlg_box_y_ordinate(height);
 
     dialog = dlg_new_window(height, width, y, x);
+    dlg_register_window(dialog, "formbox", binding);
+    dlg_register_buttons(dialog, "formbox", buttons);
 
     dlg_mouse_setbase(x, y);
 
@@ -437,16 +485,15 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 
     show_buttons = TRUE;
     scroll_changed = TRUE;
-    field_changed = TRUE;
 
-    choice = set_choice(elt, choice, item_no);
-    current = &elt[choice];
+    choice = set_choice(items, choice, item_no);
+    current = &items[choice];
 
     while (result == DLG_EXIT_UNKNOWN) {
 	int edit = FALSE;
 
 	if (scroll_changed) {
-	    print_form(form, elt, item_no, scrollamt, choice);
+	    print_form(form, items, item_no, scrollamt, choice);
 	    dlg_draw_arrows(dialog, scrollamt,
 			    scrollamt + form_height < item_no,
 			    box_x + 1,
@@ -468,18 +515,21 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 	if (field_changed || state == sTEXT) {
 	    if (field_changed)
 		chr_offset = 0;
-	    current = &elt[choice];
+	    current = &items[choice];
 	    dialog_vars.max_input = current->text_ilen;
 	    dlg_item_help(current->help);
 	    dlg_show_string(form, current->text, chr_offset,
 			    form_active_text_attr,
 			    current->text_y - scrollamt,
 			    current->text_x,
-			    current->text_len, password, first);
+			    current->text_len,
+			    isPassword(current), first);
 	    field_changed = FALSE;
 	}
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
+	if (dlg_result_key(key, fkey, &result))
+	    break;
 
 	/* handle non-functionkeys */
 	if (!fkey) {
@@ -487,39 +537,13 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 		code = dlg_char_to_button(key, buttons);
 		if (code >= 0) {
 		    dlg_del_window(dialog);
-		    result = code;
+		    result = dlg_ok_buttoncode(code);
 		    continue;
 		}
-	    }
-
-	    fkey = TRUE;
-	    switch (key) {
-	    case ESC:
-		result = DLG_EXIT_ESC;
-		continue;
-	    case CHR_NEXT:
-		key = KEY_NEXT;
-		break;
-	    case CHR_PREVIOUS:
-		key = KEY_PREVIOUS;
-		break;
-	    case ' ':
-		if (state == sTEXT) {
-		    fkey = FALSE;
-		    break;
+		if (key == ' ') {
+		    fkey = TRUE;
+		    key = DLGK_ENTER;
 		}
-		/* FALLTHRU */
-	    case '\n':
-	    case '\r':
-		key = KEY_ENTER;
-		break;
-	    case TAB:
-		state = dlg_next_ok_buttonindex(state, sTEXT);
-		show_buttons = TRUE;
-		continue;
-	    default:
-		fkey = FALSE;
-		break;
 	    }
 	}
 
@@ -530,29 +554,28 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 	    int move_by = 0;
 
 	    switch (key) {
-	    case M_EVENT + KEY_PPAGE:
-	    case KEY_PPAGE:
+	    case DLGK_MOUSE(KEY_PPAGE):
+	    case DLGK_PAGE_PREV:
 		do_scroll = TRUE;
 		move_by = -form_height;
 		break;
 
-	    case M_EVENT + KEY_NPAGE:
-	    case KEY_NPAGE:
+	    case DLGK_MOUSE(KEY_NPAGE):
+	    case DLGK_PAGE_NEXT:
 		do_scroll = TRUE;
 		move_by = form_height;
 		break;
 
-	    case KEY_ENTER:
+	    case DLGK_ENTER:
 		dlg_del_window(dialog);
 		result = (state >= 0) ? dlg_ok_buttoncode(state) : DLG_EXIT_OK;
 		continue;
 
-	    case KEY_LEFT:
+	    case DLGK_GRID_LEFT:
 		if (state == sTEXT)
 		    break;
 		/* FALLTHRU */
-	    case KEY_UP:
-	    case KEY_PREVIOUS:
+	    case DLGK_ITEM_PREV:
 		if (state == sTEXT) {
 		    do_tab = TRUE;
 		    move_by = -1;
@@ -563,18 +586,22 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 		    continue;
 		}
 
-	    case KEY_BTAB:
+	    case DLGK_FIELD_PREV:
 		state = dlg_prev_ok_buttonindex(state, sTEXT);
 		show_buttons = TRUE;
 		continue;
 
-	    case KEY_RIGHT:
+	    case DLGK_FIELD_NEXT:
+		state = dlg_next_ok_buttonindex(state, sTEXT);
+		show_buttons = TRUE;
+		continue;
+
+	    case DLGK_GRID_RIGHT:
 		if (state == sTEXT)
 		    break;
 		/* FALLTHRU */
 
-	    case KEY_DOWN:
-	    case KEY_NEXT:
+	    case DLGK_ITEM_NEXT:
 		if (state == sTEXT) {
 		    do_tab = TRUE;
 		    move_by = 1;
@@ -585,33 +612,46 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 		    continue;
 		}
 
+#ifdef KEY_RESIZE
+	    case KEY_RESIZE:
+		/* reset data */
+		height = old_height;
+		width = old_width;
+		/* repaint */
+		dlg_clear();
+		dlg_del_window(dialog);
+		refresh();
+		dlg_mouse_free_regions();
+		goto retry;
+		break;
+#endif
 	    default:
 #if USE_MOUSE
-		if (key >= M_EVENT) {
-		    if (key >= M_EVENT + KEY_MAX) {
-			int cell = key - (M_EVENT + KEY_MAX);
+		if (is_DLGK_MOUSE(key)) {
+		    if (key >= DLGK_MOUSE(KEY_MAX)) {
+			int cell = key - DLGK_MOUSE(KEY_MAX);
 			int row = (cell / getmaxx(form)) + scrollamt;
 			int col = (cell % getmaxx(form));
 			int n;
 
 			for (n = 0; n < item_no; ++n) {
-			    if (elt[n].name_y == row
-				&& elt[n].name_x <= col
-				&& (elt[n].name_x + elt[n].name_len > col
-				    || (elt[n].name_y == elt[n].text_y
-					&& elt[n].text_x > col))) {
+			    if (items[n].name_y == row
+				&& items[n].name_x <= col
+				&& (items[n].name_x + items[n].name_len > col
+				    || (items[n].name_y == items[n].text_y
+					&& items[n].text_x > col))) {
 				field_changed = TRUE;
 				break;
 			    }
-			    if (elt[n].text_y == row
-				&& elt[n].text_x <= col
-				&& elt[n].text_x + elt[n].text_ilen > col) {
+			    if (items[n].text_y == row
+				&& items[n].text_x <= col
+				&& items[n].text_x + items[n].text_ilen > col) {
 				field_changed = TRUE;
 				break;
 			    }
 			}
 			if (field_changed) {
-			    print_item(form, elt + choice, scrollamt, FALSE);
+			    print_item(form, items + choice, scrollamt, FALSE);
 			    choice = n;
 			    continue;
 			}
@@ -628,7 +668,7 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 	    new_scroll = scrollamt;
 	    new_choice = choice;
 	    if (do_scroll) {
-		if (scroll_next(form, elt, move_by, &new_choice, &new_scroll)) {
+		if (scroll_next(form, items, move_by, &new_choice, &new_scroll)) {
 		    if (choice != new_choice) {
 			choice = new_choice;
 			field_changed = TRUE;
@@ -641,7 +681,7 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 		continue;
 	    }
 	    if (do_tab) {
-		if (tab_next(form, elt, item_no, move_by, &new_choice, &new_scroll)) {
+		if (tab_next(form, items, item_no, move_by, &new_choice, &new_scroll)) {
 		    if (choice != new_choice) {
 			choice = new_choice;
 			field_changed = TRUE;
@@ -662,13 +702,98 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
 				form_active_text_attr,
 				current->text_y - scrollamt,
 				current->text_x,
-				current->text_len, password, first);
+				current->text_len,
+				isPassword(current), first);
 		continue;
 	    }
 	}
 
     }
 
+    dlg_mouse_free_regions();
+    dlg_del_window(dialog);
+    free(prompt);
+
+    *current_item = scrollamt + choice;
+    return result;
+}
+
+/*
+ * Free memory owned by a list of DIALOG_FORMITEM's.
+ */
+void
+dlg_free_formitems(DIALOG_FORMITEM * items)
+{
+    int n;
+    for (n = 0; items[n].name != 0; ++n) {
+	if (items[n].name_free)
+	    free(items[n].name);
+	if (items[n].text_free)
+	    free(items[n].text);
+	if (items[n].help_free)
+	    free(items[n].help);
+    }
+    free(items);
+}
+
+/*
+ * The script accepts values beginning at 1, while curses starts at 0.
+ */
+static int
+ordinate(const char *s)
+{
+    int result = atoi(s);
+    if (result > 0)
+	--result;
+    else
+	result = 0;
+    return result;
+}
+
+int
+dialog_form(const char *title,
+	    const char *cprompt,
+	    int height,
+	    int width,
+	    int form_height,
+	    int item_no,
+	    char **items)
+{
+    int result;
+    int choice;
+    int i;
+    DIALOG_FORMITEM *listitems;
+    bool show_status = FALSE;
+
+    listitems = calloc(item_no + 1, sizeof(*listitems));
+    assert_ptr(listitems, "dialog_menu");
+
+    for (i = 0; i < item_no; ++i) {
+	listitems[i].type = dialog_vars.formitem_type;
+	listitems[i].name = ItemName(i);
+	listitems[i].name_len = strlen(ItemName(i));
+	listitems[i].name_y = ordinate(ItemNameY(i));
+	listitems[i].name_x = ordinate(ItemNameX(i));
+	listitems[i].text = ItemText(i);
+	listitems[i].text_len = strlen(ItemText(i));
+	listitems[i].text_y = ordinate(ItemTextY(i));
+	listitems[i].text_x = ordinate(ItemTextX(i));
+	listitems[i].text_flen = atoi(ItemTextFLen(i));
+	listitems[i].text_ilen = atoi(ItemTextILen(i));
+	listitems[i].help = (dialog_vars.item_help) ? ItemHelp(i) : "";
+
+	listitems[i].text_flen = real_length(&listitems[i]);
+    }
+
+    result = dlg_form(title,
+		      cprompt,
+		      height,
+		      width,
+		      form_height,
+		      item_no,
+		      listitems,
+		      &choice);
+
     switch (result) {
     case DLG_EXIT_OK:		/* FALLTHRU */
     case DLG_EXIT_EXTRA:
@@ -677,11 +802,11 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
     case DLG_EXIT_HELP:
 	dlg_add_result("HELP ");
 	show_status = dialog_vars.help_status;
-	if (USE_ITEM_HELP(ItemHelp(scrollamt + choice))) {
-	    dlg_add_result(ItemHelp(scrollamt + choice));
+	if (USE_ITEM_HELP(listitems[choice].help)) {
+	    dlg_add_result(listitems[choice].help);
 	    result = DLG_EXIT_ITEM_HELP;
 	} else {
-	    dlg_add_result(ItemName(scrollamt + choice));
+	    dlg_add_result(listitems[choice].name);
 	}
 	if (show_status)
 	    dlg_add_result("\n");
@@ -689,18 +814,14 @@ dialog_form(const char *title, const char *cprompt, int height, int width,
     }
     if (show_status) {
 	for (i = 0; i < item_no; i++) {
-	    if (elt[i].text_flen > 0) {
-		dlg_add_result(elt[i].text);
+	    if (listitems[i].text_flen > 0) {
+		dlg_add_result(listitems[i].text);
 		dlg_add_result("\n");
-		free(elt[i].text);
 	    }
 	}
     }
 
-    free(elt);
+    dlg_free_formitems(listitems);
 
-    dlg_mouse_free_regions();
-    dlg_del_window(dialog);
-    free(prompt);
     return result;
 }
