@@ -1,26 +1,29 @@
 /*
- * $Id: fselect.c,v 1.44 2004/09/18 16:36:40 tom Exp $
+ * $Id: fselect.c,v 1.57 2005/12/07 00:41:44 tom Exp $
  *
  *  fselect.c -- implements the file-selector box
  *
- * Copyright 2000-2003,2004   Thomas E. Dickey
+ * Copyright 2000-2004,2005   Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -46,7 +49,7 @@
 #define HDR_HIGH 1
 #define BTN_HIGH (1 + 2 * MARGIN)	/* Ok/Cancel, also input-box */
 #define MIN_HIGH (HDR_HIGH - MARGIN + (BTN_HIGH * 2) + 4 * MARGIN)
-#define MIN_WIDE (2 * MAX(strlen(d_label), strlen(f_label)) + 6 * MARGIN + 2 * EXT_WIDE)
+#define MIN_WIDE (2 * MAX(dlg_count_columns(d_label), dlg_count_columns(f_label)) + 6 * MARGIN + 2 * EXT_WIDE)
 
 #define MOUSE_D (KEY_MAX + 0)
 #define MOUSE_F (KEY_MAX + 10000)
@@ -406,6 +409,28 @@ usable_state(STATES state, LIST * dirs, LIST * files)
 int
 dialog_fselect(const char *title, const char *path, int height, int width)
 {
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_LEFT ),	/* override inputstr */
+	INPUTSTR_BINDINGS,
+	ENTERKEY_BINDINGS,
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, KEY_RIGHT ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_DOWN),
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_NEXT ),
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,  KEY_UP ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  KEY_NPAGE ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,  KEY_PPAGE ),
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
+#ifdef KEY_RESIZE
+    int old_height = height;
+    int old_width = width;
+    bool resized = FALSE;
+#endif
     int tbox_y, tbox_x, tbox_width, tbox_height;
     int dbox_y, dbox_x, dbox_width, dbox_height;
     int fbox_y, fbox_x, fbox_width, fbox_height;
@@ -418,7 +443,7 @@ dialog_fselect(const char *title, const char *path, int height, int width)
     int state = dialog_vars.defaultno ? dlg_defaultno_button() : sTEXT;
     int button = state;
     int first = (state == sTEXT);
-    char *input = dialog_vars.input_result;
+    char *input;
     char *completed;
     char current[MAX_LEN + 1];
     WINDOW *dialog, *w_text, *w_dir, *w_file;
@@ -431,6 +456,14 @@ dialog_fselect(const char *title, const char *path, int height, int width)
 
     dlg_does_output();
 
+    /* Set up the initial value */
+    input = dlg_set_result(path);
+    offset = strlen(input);
+    *current = 0;
+
+#ifdef KEY_RESIZE
+  retry:
+#endif
     dlg_auto_size(title, (char *) 0, &height, &width, 6, 25);
     height += MIN_HIGH + min_items;
     if (width < min_wide)
@@ -441,6 +474,9 @@ dialog_fselect(const char *title, const char *path, int height, int width)
     dialog = dlg_new_window(height, width,
 			    dlg_box_y_ordinate(height),
 			    dlg_box_x_ordinate(width));
+    dlg_register_window(dialog, "fselect", binding);
+    dlg_register_buttons(dialog, "fselect", buttons);
+
     dlg_mouse_setbase(0, 0);
 
     dlg_draw_box(dialog, 0, 0, height, width, dialog_attr, border_attr);
@@ -505,17 +541,19 @@ dialog_fselect(const char *title, const char *path, int height, int width)
 		 menubox_border_attr, menubox_attr);
     init_list(&f_list, dialog, w_file, MOUSE_F);
 
-    /* Set up the initial value */
-    strcpy(input, path);
-    offset = strlen(input);
-    *current = 0;
-
     while (result == DLG_EXIT_UNKNOWN) {
-	int edit = 0;
 
 	if (fill_lists(current, input, &d_list, &f_list, state < sTEXT))
 	    show_buttons = TRUE;
 
+#ifdef KEY_RESIZE
+	if (resized) {
+	    resized = FALSE;
+	    dlg_show_string(w_text, input, offset, inputbox_attr,
+			    0, 0, tbox_width, 0, first);
+	}
+#endif
+
 	/*
 	 * The last field drawn determines where the cursor is shown:
 	 */
@@ -544,79 +582,60 @@ dialog_fselect(const char *title, const char *path, int height, int width)
 	    fix_arrows(&d_list);
 	    fix_arrows(&f_list);
 	    key = dlg_mouse_wgetch(dialog, &fkey);
+	    if (dlg_result_key(key, fkey, &result))
+		break;
 	}
 
-	if (!fkey) {
+	if (!fkey && key == ' ') {
+	    key = DLGK_SELECT;
 	    fkey = TRUE;
-	    switch (key) {
-	    case ESC:
-		result = DLG_EXIT_ESC;
-		fkey = FALSE;
-		break;
-	    case '\n':
-	    case '\r':
-		key = KEY_ENTER;
-		break;
-	    case ' ':
-		key = KEY_SELECT;
-		break;
-	    case TAB:
-		key = KEY_RIGHT;
-		break;
-	    case 0:
-		break;
-	    default:
-		fkey = FALSE;
-		break;
-	    }
 	}
 
 	if (fkey) {
 	    switch (key) {
-	    case M_EVENT + KEY_PREVIOUS:
+	    case DLGK_MOUSE(KEY_PREVIOUS):
 		state = sDIRS;
 		scroll_list(-1, which_list());
 		continue;
-	    case M_EVENT + KEY_NEXT:
+	    case DLGK_MOUSE(KEY_NEXT):
 		state = sDIRS;
 		scroll_list(1, which_list());
 		continue;
-	    case M_EVENT + KEY_PPAGE:
+	    case DLGK_MOUSE(KEY_PPAGE):
 		state = sFILES;
 		scroll_list(-1, which_list());
 		continue;
-	    case M_EVENT + KEY_NPAGE:
+	    case DLGK_MOUSE(KEY_NPAGE):
 		state = sFILES;
 		scroll_list(1, which_list());
 		continue;
-	    case KEY_PPAGE:
+	    case DLGK_PAGE_PREV:
 		scroll_list(-1, which_list());
 		continue;
-	    case KEY_NPAGE:
+	    case DLGK_PAGE_NEXT:
 		scroll_list(1, which_list());
 		continue;
-	    case KEY_UP:
+	    case DLGK_ITEM_PREV:
 		if (change_list(-1, which_list()))
 		    continue;
 		/* FALLTHRU */
-	    case KEY_LEFT:
-	    case KEY_BTAB:
+	    case DLGK_FIELD_PREV:
 		show_buttons = TRUE;
 		do {
 		    state = dlg_prev_ok_buttonindex(state, sDIRS);
 		} while (!usable_state(state, &d_list, &f_list));
 		continue;
-	    case KEY_DOWN:
+	    case DLGK_ITEM_NEXT:
 		if (change_list(1, which_list()))
 		    continue;
 		/* FALLTHRU */
-	    case KEY_RIGHT:
+	    case DLGK_FIELD_NEXT:
 		show_buttons = TRUE;
 		do {
 		    state = dlg_next_ok_buttonindex(state, sDIRS);
 		} while (!usable_state(state, &d_list, &f_list));
 		continue;
-	    case KEY_SELECT:
+	    case DLGK_SELECT:
 		completed = 0;
 		if (state == sFILES) {
 		    completed = data_of(&f_list);
@@ -636,24 +655,40 @@ dialog_fselect(const char *title, const char *path, int height, int width)
 		    continue;
 		}
 		/* FALLTHRU */
-	    case KEY_ENTER:
+	    case DLGK_ENTER:
 		result = (state > 0) ? dlg_ok_buttoncode(state) : DLG_EXIT_OK;
 		continue;
+#ifdef KEY_RESIZE
+	    case KEY_RESIZE:
+		/* reset data */
+		height = old_height;
+		width = old_width;
+		show_buttons = TRUE;
+		*current = 0;
+		resized = TRUE;
+		/* repaint */
+		dlg_clear();
+		dlg_del_window(dialog);
+		refresh();
+		dlg_mouse_free_regions();
+		goto retry;
+		break;
+#endif
 	    default:
-		if (key >= M_EVENT + MOUSE_T) {
+		if (key >= DLGK_MOUSE(MOUSE_T)) {
 		    state = sTEXT;
 		    continue;
-		} else if (key >= M_EVENT + MOUSE_F) {
+		} else if (key >= DLGK_MOUSE(MOUSE_F)) {
 		    state = sFILES;
-		    f_list.choice = (key - (M_EVENT + MOUSE_F)) + f_list.offset;
+		    f_list.choice = (key - DLGK_MOUSE(MOUSE_F)) + f_list.offset;
 		    display_list(&f_list);
 		    continue;
-		} else if (key >= M_EVENT + MOUSE_D) {
+		} else if (key >= DLGK_MOUSE(MOUSE_D)) {
 		    state = sDIRS;
-		    d_list.choice = (key - (M_EVENT + MOUSE_D)) + d_list.offset;
+		    d_list.choice = (key - DLGK_MOUSE(MOUSE_D)) + d_list.offset;
 		    display_list(&d_list);
 		    continue;
-		} else if (key >= M_EVENT
+		} else if (is_DLGK_MOUSE(key)
 			   && (code = dlg_ok_buttoncode(key - M_EVENT)) >= 0) {
 		    result = code;
 		    continue;
@@ -663,7 +698,7 @@ dialog_fselect(const char *title, const char *path, int height, int width)
 	}
 
 	if (state < 0) {	/* Input box selected if we're editing */
-	    edit = dlg_edit_string(input, &offset, key, fkey, first);
+	    int edit = dlg_edit_string(input, &offset, key, fkey, first);
 
 	    if (edit) {
 		dlg_show_string(w_text, input, offset, inputbox_attr,
@@ -673,7 +708,7 @@ dialog_fselect(const char *title, const char *path, int height, int width)
 	    }
 	} else if (state >= 0 &&
 		   (code = dlg_char_to_button(key, buttons)) >= 0) {
-	    result = code;
+	    result = dlg_ok_buttoncode(code);
 	    break;
 	}
     }
