@@ -1,27 +1,32 @@
 /*
- *  $Id: inputbox.c,v 1.43 2004/09/19 22:09:50 tom Exp $
+ *  $Id: inputbox.c,v 1.55 2005/12/06 20:33:19 tom Exp $
  *
  *  inputbox.c -- implements the input box
  *
- *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
- *     and: Thomas E. Dickey
+ *  Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  An earlier version of this program lists as authors:
+ *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
 
 #define sTEXT -1
 
@@ -32,15 +37,33 @@ int
 dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 		const char *init, const int password)
 {
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	INPUTSTR_BINDINGS,
+	ENTERKEY_BINDINGS,
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	KEY_DOWN ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	KEY_RIGHT ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	TAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_BTAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_LEFT ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_UP ),
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
+#ifdef KEY_RESIZE
+    int old_height = height;
+    int old_width = width;
+#endif
     int x, y, box_y, box_x, box_width;
-    int show_buttons = TRUE;
+    int show_buttons;
     int col_offset = 0;
     int chr_offset = 0;
-    int key = 0, fkey = 0, code;
+    int key, fkey, code;
     int result = DLG_EXIT_UNKNOWN;
-    int state = dialog_vars.defaultno ? dlg_defaultno_button() : sTEXT;
-    int first = (state == sTEXT);
-    char *input = dialog_vars.input_result;
+    int state;
+    int first;
+    char *input;
     WINDOW *dialog;
     char *prompt = dlg_strclone(cprompt);
     const char **buttons = dlg_ok_labels();
@@ -48,9 +71,21 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
     dlg_does_output();
 
     dlg_tab_correct_str(prompt);
+
+    /* Set up the initial value */
+    input = dlg_set_result(init);
+
+#ifdef KEY_RESIZE
+  retry:
+#endif
+    show_buttons = TRUE;
+    state = dialog_vars.defaultno ? dlg_defaultno_button() : sTEXT;
+    first = (state == sTEXT);
+    key = fkey = 0;
+
     if (init != NULL) {
 	dlg_auto_size(title, prompt, &height, &width, 5,
-		      MIN(MAX((int) strlen(init) + 7, 26),
+		      MIN(MAX(dlg_count_columns(init) + 7, 26),
 			  SCOLS - (dialog_vars.begin_set ?
 				   dialog_vars.begin_x : 0)));
     } else {
@@ -63,6 +98,8 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
     y = dlg_box_y_ordinate(height);
 
     dialog = dlg_new_window(height, width, y, x);
+    dlg_register_window(dialog, "inputbox", binding);
+    dlg_register_buttons(dialog, "inputbox", buttons);
 
     dlg_mouse_setbase(x, y);
 
@@ -82,12 +119,6 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
     dlg_draw_box(dialog, y + 1, box_x - 1, 3, box_width + 2,
 		 border_attr, dialog_attr);
 
-    /* Set up the initial value */
-    if (!init)
-	input[0] = '\0';
-    else
-	strcpy((char *) input, init);
-
     while (result == DLG_EXIT_UNKNOWN) {
 	int edit = 0;
 
@@ -101,15 +132,18 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 	    dlg_draw_buttons(dialog, height - 2, 0, buttons, state, FALSE, width);
 	}
 
-	if (!first)
+	if (!first) {
 	    key = dlg_mouse_wgetch(dialog, &fkey);
+	    if (dlg_result_key(key, fkey, &result))
+		break;
+	}
 
 	/*
 	 * Handle mouse clicks first, since we want to know if this is a button,
 	 * or something that dlg_edit_string() should handle.
 	 */
 	if (fkey
-	    && key >= M_EVENT
+	    && is_DLGK_MOUSE(key)
 	    && (code = dlg_ok_buttoncode(key - M_EVENT)) >= 0) {
 	    result = code;
 	    continue;
@@ -123,56 +157,51 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 				box_y, box_x, box_width, password, first);
 		first = FALSE;
 		continue;
+	    } else if (first) {
+		first = FALSE;
+		continue;
 	    }
 	}
 
 	/* handle non-functionkeys */
-	if (!fkey) {
-
-	    if ((code = dlg_char_to_button(key, buttons)) >= 0) {
-		dlg_del_window(dialog);
-		result = code;
-		continue;
-	    }
-
-	    switch (key) {
-	    case ESC:
-		result = DLG_EXIT_ESC;
-		break;
-	    case TAB:
-		key = KEY_RIGHT;
-		fkey = TRUE;
-		break;
-	    case ' ':
-	    case '\n':
-	    case '\r':
-		key = KEY_ENTER;
-		fkey = TRUE;
-		break;
-	    }
+	if (!fkey && (code = dlg_char_to_button(key, buttons)) >= 0) {
+	    dlg_del_window(dialog);
+	    result = dlg_ok_buttoncode(code);
+	    continue;
 	}
 
 	/* handle functionkeys */
 	if (fkey) {
 	    switch (key) {
-	    case M_EVENT + 'i':	/* mouse enter events */
+	    case DLGK_MOUSE('i'):	/* mouse enter events */
 		state = 0;
 		/* FALLTHRU */
-	    case KEY_BTAB:
-	    case KEY_UP:
-	    case KEY_LEFT:
+	    case DLGK_FIELD_PREV:
 		show_buttons = TRUE;
 		state = dlg_prev_ok_buttonindex(state, sTEXT);
 		break;
-	    case KEY_DOWN:
-	    case KEY_RIGHT:
+	    case DLGK_FIELD_NEXT:
 		show_buttons = TRUE;
 		state = dlg_next_ok_buttonindex(state, sTEXT);
 		break;
-	    case KEY_ENTER:
+	    case ' ':		/* FIXME: conflict with inputstr.c */
+	    case DLGK_ENTER:
 		dlg_del_window(dialog);
 		result = (state >= 0) ? dlg_ok_buttoncode(state) : DLG_EXIT_OK;
 		break;
+#ifdef KEY_RESIZE
+	    case KEY_RESIZE:
+		/* reset data */
+		height = old_height;
+		width = old_width;
+		/* repaint */
+		dlg_clear();
+		dlg_del_window(dialog);
+		refresh();
+		dlg_mouse_free_regions();
+		goto retry;
+		break;
+#endif
 	    default:
 		beep();
 		break;
