@@ -1,27 +1,35 @@
 /*
- *  $Id: textbox.c,v 1.57 2004/12/20 20:47:15 tom Exp $
+ *  $Id: textbox.c,v 1.77 2005/12/07 00:45:30 tom Exp $
  *
  *  textbox.c -- implements the text box
  *
- *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
- *     and: Thomas E. Dickey
+ *  Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  An earlier version of this program lists as authors:
+ *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
+
+#define PAGE_LENGTH	(height - 4)
+#define PAGE_WIDTH	(width - 2)
 
 typedef struct {
     DIALOG_CALLBACK obj;
@@ -161,7 +169,25 @@ read_high(MY_OBJ * obj, size_t size_read)
 }
 
 static long
-tabize(MY_OBJ * obj, long val, int pos)
+find_first(MY_OBJ * obj, char *buffer, long length)
+{
+    long recount = obj->page_length;
+    long result = 0;
+
+    while (length > 0) {
+	if (buffer[length] == '\n') {
+	    if (--recount < 0) {
+		result = length;
+		break;
+	    }
+	}
+	--length;
+    }
+    return result;
+}
+
+static long
+tabize(MY_OBJ * obj, long val, long *first_pos)
 {
     long fpos;
     long i, count, begin_line;
@@ -181,9 +207,12 @@ tabize(MY_OBJ * obj, long val, int pos)
 	dlg_exiterr("Error reading file in tabize().");
 
     begin_line = count = 0;
+    if (first_pos != 0)
+	*first_pos = 0;
+
     for (i = 0; i < val; i++) {
-	if (pos && count >= val) {
-	    count = i;		/* it's the retval */
+	if ((first_pos != 0) && (count >= val)) {
+	    *first_pos = find_first(obj, buftab, i);
 	    break;
 	}
 	if (buftab[i] == TAB)
@@ -238,7 +267,7 @@ get_line(MY_OBJ * obj)
     if (i <= MAX_LEN)
 	obj->line[i] = '\0';
     if (!obj->end_reached)
-	obj->in_buf++;		/* move pass '\n' */
+	obj->in_buf++;		/* move past '\n' */
 
     return obj->line;
 }
@@ -280,7 +309,7 @@ back_lines(MY_OBJ * obj, long n)
 		 */
 		/* Really possible to move backward BUF_SIZE/2 bytes? */
 		if (fpos < BUF_SIZE / 2 + obj->fd_bytes_read) {
-		    /* No, move less then */
+		    /* No, move less than */
 		    lseek_obj(obj, 0, SEEK_SET);
 		    val_to_tabize = fpos - obj->fd_bytes_read;
 		} else {	/* Move backward BUF_SIZE/2 bytes */
@@ -289,7 +318,7 @@ back_lines(MY_OBJ * obj, long n)
 		}
 		read_high(obj, BUF_SIZE);
 
-		obj->in_buf = tabize(obj, val_to_tabize, 0);
+		obj->in_buf = tabize(obj, val_to_tabize, (long *) 0);
 
 	    } else {		/* Beginning of file reached */
 		obj->begin_reached = TRUE;
@@ -311,7 +340,7 @@ back_lines(MY_OBJ * obj, long n)
 		if (fpos > obj->fd_bytes_read) {
 		    /* Really possible to move backward BUF_SIZE/2 bytes? */
 		    if (fpos < BUF_SIZE / 2 + obj->fd_bytes_read) {
-			/* No, move less then */
+			/* No, move less than */
 			lseek_obj(obj, 0, SEEK_SET);
 			val_to_tabize = fpos - obj->fd_bytes_read;
 		    } else {	/* Move backward BUF_SIZE/2 bytes */
@@ -320,7 +349,7 @@ back_lines(MY_OBJ * obj, long n)
 		    }
 		    read_high(obj, BUF_SIZE);
 
-		    obj->in_buf = tabize(obj, val_to_tabize, 0);
+		    obj->in_buf = tabize(obj, val_to_tabize, (long *) 0);
 
 		} else {	/* Beginning of file reached */
 		    obj->begin_reached = TRUE;
@@ -339,18 +368,22 @@ static void
 print_line(MY_OBJ * obj, int row, int width)
 {
     int i, y, x;
-    char *line;
+    char *line = get_line(obj);
+    const int *cols = dlg_index_columns(line);
+    const int *indx = dlg_index_wchars(line);
+    int limit = dlg_count_wchars(line);
+    int first = 0;
+    int last = limit;
+
+    for (i = 0; i <= limit && cols[i] < obj->hscroll; ++i)
+	first = i;
+
+    for (i = first; i <= limit && cols[i] - cols[first] <= PAGE_WIDTH; ++i)
+	last = i;
 
-    line = get_line(obj);
-    line += MIN((int) strlen(line), obj->hscroll);	/* Scroll horizontally */
     (void) wmove(obj->text, row, 0);	/* move cursor to correct line */
     (void) waddch(obj->text, ' ');
-#ifdef NCURSES_VERSION
-    (void) waddnstr(obj->text, line, MIN((int) strlen(line), width - 2));
-#else
-    line[MIN((int) strlen(line), width - 2)] = '\0';
-    waddstr(obj->text, line);
-#endif
+    (void) waddnstr(obj->text, line + indx[first], indx[last] - indx[first]);
 
     getyx(obj->text, y, x);
     /* Clear 'residue' of previous line */
@@ -388,10 +421,14 @@ print_position(MY_OBJ * obj, WINDOW *win, int height, int width)
     long size;
     int percent;
     int len;
-    char buffer[5];
+    long first = -1;
+    char buffer[80];
 
     fpos = ftell_obj(obj);
-    size = tabize(obj, obj->in_buf, 1);
+    if (dialog_vars.tab_correct)
+	size = tabize(obj, obj->in_buf, &first);
+    else
+	first = find_first(obj, obj->buf, size = obj->in_buf);
 
     wattrset(win, position_indicator_attr);
     percent = (!obj->file_size
@@ -405,19 +442,19 @@ print_position(MY_OBJ * obj, WINDOW *win, int height, int width)
 	percent = 100;
 
     (void) sprintf(buffer, "%d%%", percent);
-    (void) wmove(win, height - 3, width - 9);
+    (void) wmove(win, PAGE_LENGTH + 1, PAGE_WIDTH - 7);
     (void) waddstr(win, buffer);
-    if ((len = strlen(buffer)) < 4) {
+    if ((len = dlg_count_columns(buffer)) < 4) {
 	wattrset(win, border_attr);
 	whline(win, ACS_HLINE, 4 - len);
     }
 
     wattrset(win, save);
     dlg_draw_arrows2(win,
-		     fpos > obj->fd_bytes_read - size,
+		     first != 0,
 		     size < obj->fd_bytes_read,
-		     5, 0, height - 3,
-		     dialog_attr,
+		     5, 0, PAGE_LENGTH + 1,
+		     border_attr,
 		     border_attr);
 }
 
@@ -427,10 +464,19 @@ print_position(MY_OBJ * obj, WINDOW *win, int height, int width)
 static int
 get_search_term(WINDOW *dialog, char *input, int height, int width)
 {
-    chtype save = getattrs(dialog);
-    int box_x, box_y, key = 0;
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	INPUTSTR_BINDINGS,
+	ENTERKEY_BINDINGS,
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
+    int old_x, old_y;
+    int box_x, box_y;
     int box_height, box_width;
     int offset = 0;
+    int key = 0;
     int fkey = 0;
     bool first = TRUE;
     int result = DLG_EXIT_UNKNOWN;
@@ -438,6 +484,9 @@ get_search_term(WINDOW *dialog, char *input, int height, int width)
     int len_caption = dlg_count_columns(caption);
     const int *indx;
     int limit;
+    WINDOW *widget;
+
+    getbegyx(dialog, old_y, old_x);
 
     box_height = 1 + (2 * MARGIN);
     box_width = len_caption + (2 * (MARGIN + 2));
@@ -451,15 +500,19 @@ get_search_term(WINDOW *dialog, char *input, int height, int width)
     if (dialog_state.use_shadow)
 	dlg_draw_shadow(dialog, box_y, box_x, box_height, box_width);
 #endif
-    dlg_draw_box(dialog, box_y, box_x, box_height, box_width,
+    widget = newwin(box_height, box_width, old_y + box_y, old_x + box_x);
+    keypad(widget, TRUE);
+    dlg_register_window(widget, "searchbox", binding);
+
+    dlg_draw_box(widget, 0, 0, box_height, box_width,
 		 searchbox_attr,
 		 searchbox_border_attr);
-    wattrset(dialog, searchbox_title_attr);
-    (void) wmove(dialog, box_y, (width - len_caption) / 2);
+    wattrset(widget, searchbox_title_attr);
+    (void) wmove(widget, 0, (box_width - len_caption) / 2);
 
     indx = dlg_index_wchars(caption);
     limit = dlg_limit_columns(caption, len_caption, 0);
-    (void) waddnstr(dialog, caption + indx[0], indx[limit] - indx[0]);
+    (void) waddnstr(widget, caption + indx[0], indx[limit] - indx[0]);
 
     box_y++;
     box_x++;
@@ -468,22 +521,22 @@ get_search_term(WINDOW *dialog, char *input, int height, int width)
 
     for (;;) {
 	if (!first) {
-	    key = dlg_getc(dialog, &fkey);
+	    key = dlg_getc(widget, &fkey);
 	    if (key == ESC) {
 		result = DLG_EXIT_ESC;
 		break;
-	    } else if (key == '\n') {
+	    } else if (key == DLGK_ENTER) {
 		result = DLG_EXIT_OK;
 		break;
 	    }
 	}
 	if (dlg_edit_string(input, &offset, key, fkey, first)) {
-	    dlg_show_string(dialog, input, offset, searchbox_attr,
-			    box_y, box_x, box_width, FALSE, first);
+	    dlg_show_string(widget, input, offset, searchbox_attr,
+			    1, 1, box_width, FALSE, first);
 	    first = FALSE;
 	}
     }
-    wattrset(dialog, save);
+    delwin(widget);
     return result;
 }
 
@@ -507,8 +560,8 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 	    }
 	    /* Get search term from user */
 	} else if (get_search_term(obj->text, search_term,
-				   height - 4,
-				   width - 2) == DLG_EXIT_ESC
+				   PAGE_LENGTH,
+				   PAGE_WIDTH) == DLG_EXIT_ESC
 		   || search_term[0] == '\0') {
 	    /* ESC pressed, or no search term, reprint page to clear box */
 	    wattrset(obj->text, dialog_attr);
@@ -571,22 +624,54 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 int
 dialog_textbox(const char *title, const char *file, int height, int width)
 {
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	ENTERKEY_BINDINGS,
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,  'J' ),
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,  'j' ),
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,  KEY_DOWN ),
+	DLG_KEYS_DATA( DLGK_GRID_LEFT,  'H' ),
+	DLG_KEYS_DATA( DLGK_GRID_LEFT,  'h' ),
+	DLG_KEYS_DATA( DLGK_GRID_LEFT,  KEY_LEFT ),
+	DLG_KEYS_DATA( DLGK_GRID_RIGHT, 'L' ),
+	DLG_KEYS_DATA( DLGK_GRID_RIGHT, 'l' ),
+	DLG_KEYS_DATA( DLGK_GRID_RIGHT, KEY_RIGHT ),
+	DLG_KEYS_DATA( DLGK_GRID_UP,    'K' ),
+	DLG_KEYS_DATA( DLGK_GRID_UP,    'k' ),
+	DLG_KEYS_DATA( DLGK_GRID_UP,    KEY_UP ),
+	DLG_KEYS_DATA( DLGK_PAGE_FIRST, 'g' ),
+	DLG_KEYS_DATA( DLGK_PAGE_FIRST, KEY_HOME ),
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,  'G' ),
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,  KEY_END ),
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,  KEY_LL ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  ' ' ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  KEY_NPAGE ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,  'B' ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,  'b' ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,  KEY_PPAGE ),
+	DLG_KEYS_DATA( DLGK_BEGIN,	'0' ),
+	DLG_KEYS_DATA( DLGK_BEGIN,	KEY_BEG ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
+#ifdef KEY_RESIZE
+    int old_height = height;
+    int old_width = width;
+#endif
     long fpos;
     int x, y, cur_x, cur_y;
     int key = 0, fkey;
     int next = 0;
-#ifdef NCURSES_VERSION
-    int i, passed_end;
-#endif
+    int i, code, passed_end;
     char search_term[MAX_LEN + 1];
     MY_OBJ obj;
     WINDOW *dialog;
-    bool moved = TRUE;
+    bool moved;
     int result = DLG_EXIT_UNKNOWN;
-
-    dlg_auto_sizefile(title, file, &height, &width, 2, 12);
-    dlg_print_size(height, width);
-    dlg_ctl_size(height, width);
+    int button = dialog_vars.extra_button ? dlg_defaultno_button() : 0;
 
     search_term[0] = '\0';	/* no search term entered yet */
 
@@ -610,27 +695,38 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 
     read_high(&obj, BUF_SIZE);
 
+#ifdef KEY_RESIZE
+  retry:
+#endif
+    moved = TRUE;
+
+    dlg_auto_sizefile(title, file, &height, &width, 2, 12);
+    dlg_print_size(height, width);
+    dlg_ctl_size(height, width);
+
     x = dlg_box_x_ordinate(width);
     y = dlg_box_y_ordinate(height);
 
     dialog = dlg_new_window(height, width, y, x);
+    dlg_register_window(dialog, "textbox", binding);
+    dlg_register_buttons(dialog, "textbox", obj.buttons);
 
     dlg_mouse_setbase(x, y);
 
     /* Create window for text region, used for scrolling text */
-    obj.text = dlg_sub_window(dialog, height - 4, width - 2, y + 1, x + 1);
+    obj.text = dlg_sub_window(dialog, PAGE_LENGTH, PAGE_WIDTH, y + 1, x + 1);
 
     /* register the new window, along with its borders */
-    dlg_mouse_mkbigregion(0, 0, height - 2, width, KEY_MAX, 1, 1, 3 /* cells */ );
+    dlg_mouse_mkbigregion(0, 0, PAGE_LENGTH + 2, width, KEY_MAX, 1, 1, 1 /* lines */ );
     dlg_draw_box(dialog, 0, 0, height, width, dialog_attr, border_attr);
     dlg_draw_bottom_box(dialog);
     dlg_draw_title(dialog, title);
 
-    dlg_draw_buttons(dialog, height - 2, 0, obj.buttons, FALSE, FALSE, width);
+    dlg_draw_buttons(dialog, PAGE_LENGTH + 2, 0, obj.buttons, button, FALSE, width);
     (void) wnoutrefresh(dialog);
     getyx(dialog, cur_y, cur_x);	/* Save cursor position */
 
-    dlg_attr_clear(obj.text, height - 4, width - 2, dialog_attr);
+    dlg_attr_clear(obj.text, PAGE_LENGTH, PAGE_WIDTH, dialog_attr);
 
     while (result == DLG_EXIT_UNKNOWN) {
 
@@ -643,16 +739,9 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		(void) scrollok(obj.text, TRUE);
 		(void) scroll(obj.text);	/* Scroll text region up one line */
 		(void) scrollok(obj.text, FALSE);
-		print_line(&obj, height - 5, width - 2);
-#ifndef NCURSES_VERSION
-		wmove(obj.text, height - 5, 0);
-		waddch(obj.text, ' ');
-		wmove(obj.text, height - 5, width - 3);
-		waddch(obj.text, ' ');
-#endif
+		print_line(&obj, PAGE_LENGTH - 1, PAGE_WIDTH);
 		(void) wnoutrefresh(obj.text);
 	    } else if (next > 0) {
-#ifdef NCURSES_VERSION
 		/*
 		 * We don't call print_page() here but use scrolling to ensure
 		 * faster screen update.  However, 'end_reached' and
@@ -665,9 +754,9 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		(void) scrollok(obj.text, FALSE);
 		obj.page_length = 0;
 		passed_end = 0;
-		for (i = 0; i < height - 4; i++) {
+		for (i = 0; i < PAGE_LENGTH; i++) {
 		    if (!i) {
-			print_line(&obj, 0, width - 2);		/* print first line of page */
+			print_line(&obj, 0, PAGE_WIDTH);	/* print first line of page */
 			(void) wnoutrefresh(obj.text);
 		    } else
 			(void) get_line(&obj);	/* Called to update 'end_reached' and 'in_buf' */
@@ -676,11 +765,8 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    if (obj.end_reached && !passed_end)
 			passed_end = 1;
 		}
-#else
-		print_page(&obj, height - 4, width - 2);
-#endif
 	    } else {
-		print_page(&obj, height - 4, width - 2);
+		print_page(&obj, PAGE_LENGTH, PAGE_WIDTH);
 	    }
 	    print_position(&obj, dialog, height, width);
 	    (void) wmove(dialog, cur_y, cur_x);		/* Restore cursor position */
@@ -690,78 +776,47 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 	next = 0;		/* ...but not scroll by a line */
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
+	if (dlg_result_key(key, fkey, &result))
+	    break;
 
-	if (!fkey) {
-	    fkey = TRUE;
-	    switch (key) {
-	    case ESC:
-		result = DLG_EXIT_ESC;
-		continue;
-	    case '\n':
-	    case '\r':
-		key = KEY_ENTER;
-		break;
-	    case ' ':
-		key = KEY_NPAGE;
-		break;
-	    case '0':
-		key = KEY_BEG;
-		break;
-	    case 'B':
-	    case 'b':
-		key = KEY_PPAGE;
-		break;
-	    case 'g':
-		key = KEY_HOME;
-		break;
-	    case 'G':
-		key = KEY_END;
-		break;
-	    case 'H':
-	    case 'h':
-		key = KEY_LEFT;
-		break;
-	    case 'K':
-	    case 'k':
-		key = KEY_UP;
-		break;
-	    case 'J':
-	    case 'j':
-		key = KEY_DOWN;
-		break;
-	    case 'L':
-	    case 'l':
-		key = KEY_RIGHT;
-		break;
-	    case '/':		/* Forward search */
-	    case 'n':		/* Repeat forward search */
-	    case '?':		/* Backward search */
-	    case 'N':		/* Repeat backward search */
-		moved = perform_search(&obj, height, width, key, search_term);
-		fkey = FALSE;
-		break;
-	    default:
-		if (dlg_char_to_button(key, obj.buttons) == 0) {
-		    key = KEY_ENTER;
-		} else {
-		    fkey = FALSE;
-		}
-		break;
-	    }
+	if (!fkey && (code = dlg_char_to_button(key, obj.buttons)) >= 0) {
+	    result = dlg_ok_buttoncode(code);
+	    break;
 	}
 
 	if (fkey) {
 	    switch (key) {
 	    default:
-		if (key >= M_EVENT)
-		    result = DLG_EXIT_OK;
-		else
+		if (is_DLGK_MOUSE(key)) {
+		    result = dlg_exit_buttoncode(key - M_EVENT);
+		    if (result < 0)
+			result = DLG_EXIT_OK;
+		} else {
 		    beep();
+		}
 		break;
-	    case KEY_ENTER:
-		result = DLG_EXIT_OK;
+	    case DLGK_FIELD_NEXT:
+		button = dlg_next_button(obj.buttons, button);
+		if (button < 0)
+		    button = 0;
+		dlg_draw_buttons(dialog,
+				 height - 2, 0,
+				 obj.buttons, button,
+				 FALSE, width);
 		break;
-	    case KEY_HOME:	/* First page */
+	    case DLGK_FIELD_PREV:
+		button = dlg_prev_button(obj.buttons, button);
+		if (button < 0)
+		    button = 0;
+		dlg_draw_buttons(dialog,
+				 height - 2, 0,
+				 obj.buttons, button,
+				 FALSE, width);
+		break;
+	    case DLGK_ENTER:
+		result = dlg_exit_buttoncode(button);
+		break;
+	    case DLGK_PAGE_FIRST:
 		if (!obj.begin_reached) {
 		    obj.begin_reached = 1;
 		    /* First page not in buffer? */
@@ -777,8 +832,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_END:	/* Last page */
-	    case KEY_LL:
+	    case DLGK_PAGE_LAST:
 		obj.end_reached = TRUE;
 		/* Last page not in buffer? */
 		fpos = ftell_obj(&obj);
@@ -790,38 +844,38 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    read_high(&obj, BUF_SIZE);
 		}
 		obj.in_buf = obj.bytes_read;
-		back_lines(&obj, height - 4);
+		back_lines(&obj, PAGE_LENGTH);
 		moved = TRUE;
 		break;
-	    case KEY_UP:	/* Previous line */
+	    case DLGK_GRID_UP:	/* Previous line */
 		if (!obj.begin_reached) {
 		    back_lines(&obj, obj.page_length + 1);
 		    next = 1;
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_PPAGE:	/* Previous page */
-	    case M_EVENT + KEY_PPAGE:
+	    case DLGK_PAGE_PREV:	/* Previous page */
+	    case DLGK_MOUSE(KEY_PPAGE):
 		if (!obj.begin_reached) {
-		    back_lines(&obj, obj.page_length + height - 4);
+		    back_lines(&obj, obj.page_length + PAGE_LENGTH);
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_DOWN:	/* Next line */
+	    case DLGK_GRID_DOWN:	/* Next line */
 		if (!obj.end_reached) {
 		    obj.begin_reached = 0;
 		    next = -1;
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_NPAGE:	/* Next page */
-	    case M_EVENT + KEY_NPAGE:
+	    case DLGK_PAGE_NEXT:	/* Next page */
+	    case DLGK_MOUSE(KEY_NPAGE):
 		if (!obj.end_reached) {
 		    obj.begin_reached = 0;
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_BEG:	/* Beginning of line */
+	    case DLGK_BEGIN:	/* Beginning of line */
 		if (obj.hscroll > 0) {
 		    obj.hscroll = 0;
 		    /* Reprint current page to scroll horizontally */
@@ -829,7 +883,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_LEFT:	/* Scroll left */
+	    case DLGK_GRID_LEFT:	/* Scroll left */
 		if (obj.hscroll > 0) {
 		    obj.hscroll--;
 		    /* Reprint current page to scroll horizontally */
@@ -837,7 +891,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    moved = TRUE;
 		}
 		break;
-	    case KEY_RIGHT:	/* Scroll right */
+	    case DLGK_GRID_RIGHT:	/* Scroll right */
 		if (obj.hscroll < MAX_LEN) {
 		    obj.hscroll++;
 		    /* Reprint current page to scroll horizontally */
@@ -845,9 +899,35 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    moved = TRUE;
 		}
 		break;
+#ifdef KEY_RESIZE
+	    case KEY_RESIZE:
+		/* reset data */
+		height = old_height;
+		width = old_width;
+		back_lines(&obj, obj.page_length);
+		moved = TRUE;
+		/* repaint */
+		dlg_clear();
+		dlg_del_window(dialog);
+		refresh();
+		dlg_mouse_free_regions();
+		goto retry;
+		break;
+#endif
 	    }
 	} else {
-	    beep();
+	    switch (key) {
+	    case '/':		/* Forward search */
+	    case 'n':		/* Repeat forward search */
+	    case '?':		/* Backward search */
+	    case 'N':		/* Repeat backward search */
+		moved = perform_search(&obj, height, width, key, search_term);
+		fkey = FALSE;
+		break;
+	    default:
+		beep();
+		break;
+	    }
 	}
     }
 
