@@ -1,27 +1,32 @@
 /*
- *  $Id: util.c,v 1.142 2004/12/22 01:17:02 tom Exp $
+ *  $Id: util.c,v 1.163 2005/12/20 00:24:59 tom Exp $
  *
  *  util.c -- miscellaneous utilities for dialog
  *
- *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
- *     and: Thomas E. Dickey
+ *  Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  An earlier version of this program lists as authors
+ *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
 
 #include <stdarg.h>
 #include <sys/types.h>
@@ -71,7 +76,10 @@ DIALOG_VARS dialog_vars;
 
 #ifdef HAVE_COLOR
 #include <dlg_colors.h>
-#define COLOR_DATA(upr) , concat(upr,_FG), concat(upr,_BG), concat(upr,_HL)
+#define COLOR_DATA(upr) , \
+	concat(DLGC_FG_,upr), \
+	concat(DLGC_BG_,upr), \
+	concat(DLGC_HL_,upr)
 #else
 #define COLOR_DATA(upr)		/*nothing */
 #endif
@@ -129,11 +137,12 @@ dlg_put_backtitle(void)
 
     if (dialog_vars.backtitle != NULL) {
 	chtype attr = A_NORMAL;
+	int backwidth = dlg_count_columns(dialog_vars.backtitle);
 
 	wattrset(stdscr, screen_attr);
 	(void) wmove(stdscr, 0, 1);
 	dlg_print_text(stdscr, dialog_vars.backtitle, COLS - 2, &attr);
-	for (i = 0; i < COLS - (int) strlen(dialog_vars.backtitle); i++)
+	for (i = 0; i < COLS - backwidth; i++)
 	    (void) waddch(stdscr, ' ');
 	(void) wmove(stdscr, 1, 1);
 	for (i = 0; i < COLS - 2; i++)
@@ -252,7 +261,7 @@ init_dialog(FILE *input, FILE *output)
     if (fileno(input) != fileno(stdin)) {
 	if ((fd1 = dup(fileno(input))) >= 0
 	    && (fd2 = dup(fileno(stdin))) >= 0) {
-	    *stdin = *fdopen(fd1, "r");
+	    (void) dup2(fileno(input), fileno(stdin));
 	    dialog_state.pipe_input = fdopen(fd2, "r");
 	    if (fileno(stdin) != 0)	/* some functions may read fd #0 */
 		(void) dup2(fileno(stdin), 0);
@@ -262,7 +271,8 @@ init_dialog(FILE *input, FILE *output)
 	if ((fd1 = open_terminal(&device, O_RDONLY)) >= 0
 	    && (fd2 = dup(fileno(stdin))) >= 0) {
 	    dialog_state.pipe_input = fdopen(fd2, "r");
-	    *stdin = *freopen(device, "r", stdin);
+	    if (freopen(device, "r", stdin) == 0)
+		dlg_exiterr("cannot open tty-input");
 	    if (fileno(stdin) != 0)	/* some functions may read fd #0 */
 		(void) dup2(fileno(stdin), 0);
 	}
@@ -474,7 +484,7 @@ end_dialog(void)
 static int
 centered(int width, const char *string)
 {
-    int len = strlen(string);
+    int len = dlg_count_columns(string);
     int left;
     int hide = 0;
     int n;
@@ -493,15 +503,22 @@ centered(int width, const char *string)
 }
 
 /*
- * Print up to 'len' bytes from 'text', optionally rendering our escape
+ * Print up to 'cols' columns from 'text', optionally rendering our escape
  * sequence for attributes and color.
  */
 void
-dlg_print_text(WINDOW *win, const char *txt, int len, chtype *attr)
+dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 {
+    int y_origin, x_origin;
+    int y_before, x_before = 0;
+    int y_after, x_after;
+    int tabbed = 0;
+    bool thisTab;
+    bool ended = FALSE;
     chtype useattr;
 
-    while (len > 0 && (*txt != '\0')) {
+    getyx(win, y_origin, x_origin);
+    while (cols > 0 && (*txt != '\0')) {
 	if (dialog_vars.colors) {
 	    while (isOurEscape(txt)) {
 		int code;
@@ -545,9 +562,9 @@ dlg_print_text(WINDOW *win, const char *txt, int len, chtype *attr)
 		}
 		++txt;
 	    }
-	    if (*txt == '\n' || *txt == '\0')
-		break;
 	}
+	if (ended || *txt == '\n' || *txt == '\0')
+	    break;
 	useattr = (*attr) & A_ATTRIBUTES;
 #ifdef HAVE_COLOR
 	/*
@@ -567,8 +584,22 @@ dlg_print_text(WINDOW *win, const char *txt, int len, chtype *attr)
 	    }
 	}
 #endif
+	/*
+	 * Write the character, using curses to tell exactly how wide it
+	 * is.  If it is a tab, discount that, since the caller thinks
+	 * tabs are nonprinting, and curses will expand tabs to one or
+	 * more blanks.
+	 */
+	thisTab = (CharOf(*txt) == TAB);
+	if (thisTab)
+	    getyx(win, y_before, x_before);
 	(void) waddch(win, CharOf(*txt++) | useattr);
-	--len;
+	getyx(win, y_after, x_after);
+	if (thisTab && (y_after == y_origin))
+	    tabbed += (x_after - x_before);
+	if (y_after != y_origin || x_after >= cols + tabbed + x_origin) {
+	    ended = TRUE;
+	}
     }
 }
 
@@ -585,63 +616,65 @@ print_line(WINDOW *win, chtype *attr, const char *prompt, int lm, int rm, int *x
     const char *test_ptr = prompt;
     const int *cols = dlg_index_columns(prompt);
     const int *indx = dlg_index_wchars(prompt);
+    int wrap_inx = 0;
+    int test_inx = 0;
     int cur_x = lm;
     int hidden = 0;
     int limit = dlg_count_wchars(prompt);
     int n;
+    int tabbed = 0;
 
     *x = 1;
 
     /*
      * Set *test_ptr to the end of the line or the right margin (rm), whichever
-     * is less, and set *wrap_ptr to the end of the last word in the line.
+     * is less, and set wrap_ptr to the end of the last word in the line.
      */
     for (n = 0; n < limit; ++n) {
+	test_ptr = prompt + indx[test_inx];
 	if (*test_ptr == '\n' || *test_ptr == '\0' || cur_x >= (rm + hidden))
 	    break;
-	if (*test_ptr == ' ' && n != 0 && prompt[indx[n - 1]] != ' ') {
-	    wrap_ptr = prompt + indx[n];
+	if (*test_ptr == '\t' && n == 0) {
+	    tabbed = 8;		/* workaround for leading tabs */
+	} else if (*test_ptr == ' ' && n != 0 && prompt[indx[n - 1]] != ' ') {
+	    wrap_inx = n;
 	    *x = cur_x;
 	} else if (isOurEscape(test_ptr)) {
 	    hidden += 3;
 	    n += 2;
 	}
-	cur_x = lm + cols[n + 1];
+	cur_x = lm + tabbed + cols[n + 1];
 	if (cur_x > (rm + hidden))
 	    break;
-	test_ptr = prompt + indx[n + 1];
+	test_inx = n + 1;
     }
 
     /*
      * If the line doesn't reach the right margin in the middle of a word, then
      * we don't have to wrap it at the end of the previous word.
      */
+    test_ptr = prompt + indx[test_inx];
     if (*test_ptr == '\n' || *test_ptr == ' ' || *test_ptr == '\0') {
-	int i = 0;
-	while (&wrap_ptr[++i] < test_ptr) {
-	    ;
+	wrap_inx = test_inx;
+	while (wrap_inx > 0 && prompt[indx[wrap_inx - 1]] == ' ') {
+	    wrap_inx--;
 	}
-	while (wrap_ptr[i - 1] == ' ') {
-	    i--;
-	}
-	wrap_ptr += i;
-	*x += i;
-    }
-
-    /*
-     * If the line has no spaces, then wrap it anyway at the right margin
-     */
-    else if (*x == 1 && cur_x >= rm) {
+	*x += indx[wrap_inx];
+    } else if (*x == 1 && cur_x >= rm) {
+	/*
+	 * If the line has no spaces, then wrap it anyway at the right margin
+	 */
 	*x = rm;
-	wrap_ptr = test_ptr;
+	wrap_inx = test_inx;
     }
+    wrap_ptr = prompt + indx[wrap_inx];
 
     /*
      * Print the line if we have a window pointer.  Otherwise this routine
      * is just being called for sizing the window.
      */
     if (win) {
-	dlg_print_text(win, prompt, (wrap_ptr - prompt - hidden), attr);
+	dlg_print_text(win, prompt, (cols[wrap_inx] - hidden), attr);
     }
 
     /* *x tells the calling function how long the line was */
@@ -816,7 +849,7 @@ real_auto_size(const char *title,
 {
     int x = (dialog_vars.begin_set ? dialog_vars.begin_x : 2);
     int y = (dialog_vars.begin_set ? dialog_vars.begin_y : 1);
-    int title_length = title ? strlen(title) : 0;
+    int title_length = title ? dlg_count_columns(title) : 0;
     int nc = 4;
     int high;
     int wide;
@@ -841,7 +874,7 @@ real_auto_size(const char *title,
     } else if (prompt != 0) {
 	wide = MAX(title_length, mincols);
 	if (strchr(prompt, '\n') == 0) {
-	    double val = dialog_state.aspect_ratio * strlen(prompt);
+	    double val = dialog_state.aspect_ratio * dlg_count_columns(prompt);
 	    int tmp = sqrt(val);
 	    wide = MAX(wide, tmp);
 	    wide = MAX(wide, longest_word(prompt));
@@ -906,7 +939,10 @@ dlg_auto_sizefile(const char *title,
 		  int boxlines,
 		  int mincols)
 {
-    int count = 0, len = title ? strlen(title) : 0, nc = 4, numlines = 2;
+    int count = 0;
+    int len = title ? dlg_count_columns(title) : 0;
+    int nc = 4;
+    int numlines = 2;
     long offset;
     int ch;
     FILE *fd;
@@ -1080,7 +1116,7 @@ dlg_exit(int code)
     }
 #ifdef NO_LEAKS
     _dlg_inputstr_leaks();
-#ifdef NCURSES_VERSION
+#if defined(NCURSES_VERSION) && defined(HAVE__NC_FREE_AND_EXIT)
     _nc_free_and_exit(code);
 #endif
 #endif
@@ -1143,17 +1179,19 @@ dlg_ctl_size(int height, int width)
     }
 }
 
+/*
+ * If the --tab-correct was not selected, convert tabs to single spaces.
+ */
 void
 dlg_tab_correct_str(char *prompt)
 {
     char *ptr;
 
-    if (!dialog_vars.tab_correct)
-	return;
-
-    while ((ptr = strchr(prompt, TAB)) != NULL) {
-	*ptr = ' ';
-	prompt = ptr;
+    if (dialog_vars.tab_correct) {
+	while ((ptr = strchr(prompt, TAB)) != NULL) {
+	    *ptr = ' ';
+	    prompt = ptr;
+	}
     }
 }
 
@@ -1171,14 +1209,28 @@ dlg_calc_listh(int *height, int *list_height, int item_no)
     (*height) += (*list_height);
 }
 
+/* obsolete */
 int
 dlg_calc_listw(int item_no, char **items, int group)
 {
     int n, i, len1 = 0, len2 = 0;
     for (i = 0; i < (item_no * group); i += group) {
-	if ((n = strlen(items[i])) > len1)
+	if ((n = dlg_count_columns(items[i])) > len1)
 	    len1 = n;
-	if ((n = strlen(items[i + 1])) > len2)
+	if ((n = dlg_count_columns(items[i + 1])) > len2)
+	    len2 = n;
+    }
+    return len1 + len2;
+}
+
+int
+dlg_calc_list_width(int item_no, DIALOG_LISTITEM * items)
+{
+    int n, i, len1 = 0, len2 = 0;
+    for (i = 0; i < item_no; ++i) {
+	if ((n = dlg_count_columns(items[i].name)) > len1)
+	    len1 = n;
+	if ((n = dlg_count_columns(items[i].text)) > len2)
 	    len2 = n;
     }
     return len1 + len2;
@@ -1285,6 +1337,7 @@ dlg_del_window(WINDOW *win)
 	    if (p->shadow != 0)
 		delwin(p->shadow);
 	    delwin(p->normal);
+	    dlg_unregister_window(p->normal);
 	    if (r == 0) {
 		dialog_state.all_windows = q;
 	    } else {
@@ -1348,19 +1401,43 @@ dlg_sub_window(WINDOW *parent, int height, int width, int y, int x)
     return win;
 }
 
+/* obsolete */
 int
 dlg_default_item(char **items, int llen)
 {
+    int result = 0;
+
     if (dialog_vars.default_item != 0) {
 	int count = 0;
 	while (*items != 0) {
-	    if (!strcmp(dialog_vars.default_item, *items))
-		return count;
+	    if (!strcmp(dialog_vars.default_item, *items)) {
+		result = count;
+		break;
+	    }
 	    items += llen;
 	    count++;
 	}
     }
-    return 0;
+    return result;
+}
+
+int
+dlg_default_listitem(DIALOG_LISTITEM * items)
+{
+    int result = 0;
+
+    if (dialog_vars.default_item != 0) {
+	int count = 0;
+	while (items->name != 0) {
+	    if (!strcmp(dialog_vars.default_item, items->name)) {
+		result = count;
+		break;
+	    }
+	    ++items;
+	    count++;
+	}
+    }
+    return result;
 }
 
 /*
@@ -1371,12 +1448,21 @@ dlg_item_help(char *txt)
 {
     if (USE_ITEM_HELP(txt)) {
 	chtype attr = A_NORMAL;
+	int y, x;
 
 	wattrset(stdscr, itemhelp_attr);
 	(void) wmove(stdscr, LINES - 1, 0);
 	(void) wclrtoeol(stdscr);
 	(void) addch(' ');
 	dlg_print_text(stdscr, txt, COLS - 1, &attr);
+	if (itemhelp_attr & A_COLOR) {
+	    /* fill the remainder of the line with the window's attributes */
+	    getyx(stdscr, y, x);
+	    while (x < COLS) {
+		(void) addch(' ');
+		++x;
+	    }
+	}
 	(void) wnoutrefresh(stdscr);
     }
 }
@@ -1510,18 +1596,71 @@ dlg_set_focus(WINDOW *parent, WINDOW *win)
 }
 
 /*
- * Most of the time we can copy into a fixed buffer.  This handles the other
- * cases, e.g., checklist.c
+ * Free storage used for the result buffer.
+ */
+void
+dlg_clr_result(void)
+{
+    if (dialog_vars.input_length) {
+	dialog_vars.input_length = 0;
+	if (dialog_vars.input_result)
+	    free(dialog_vars.input_result);
+    }
+    dialog_vars.input_result = 0;
+}
+
+/*
+ * Setup a fixed-buffer for the result.
+ */
+char *
+dlg_set_result(const char *string)
+{
+    unsigned need = string ? strlen(string) + 1 : 0;
+
+    /* inputstr.c needs a fixed buffer */
+    if (need < MAX_LEN)
+	need = MAX_LEN;
+
+    /*
+     * If the buffer is not big enough, allocate a new one.
+     */
+    if (dialog_vars.input_length != 0
+	|| dialog_vars.input_result == 0
+	|| need > MAX_LEN) {
+
+	dlg_clr_result();
+
+	dialog_vars.input_length = need;
+	dialog_vars.input_result = malloc(need);
+	assert_ptr(dialog_vars.input_result, "dlg_set_result");
+    }
+
+    strcpy(dialog_vars.input_result, string ? string : "");
+
+    return dialog_vars.input_result;
+}
+
+/*
+ * Accumulate results in dynamically allocated buffer.
+ * If input_length is zero, it is a MAX_LEN buffer belonging to the caller.
  */
 void
 dlg_add_result(char *string)
 {
-    unsigned have = strlen(dialog_vars.input_result);
-    unsigned want = strlen(string) + have;
+    unsigned have = (dialog_vars.input_result
+		     ? strlen(dialog_vars.input_result)
+		     : 0);
+    unsigned want = strlen(string) + 1 + have;
+
+    if ((want >= MAX_LEN)
+	|| (dialog_vars.input_length != 0)
+	|| (dialog_vars.input_result == 0)) {
+
+	if (dialog_vars.input_length == 0
+	    || dialog_vars.input_result == 0) {
 
-    if (want >= MAX_LEN) {
-	if (dialog_vars.input_length == 0) {
 	    char *save = dialog_vars.input_result;
+
 	    dialog_vars.input_length = want * 2;
 	    dialog_vars.input_result = malloc(dialog_vars.input_length);
 	    assert_ptr(dialog_vars.input_result, "dlg_add_result malloc");
